program CPFlowsheets;

{%TogetherDiagram 'ModelSupport_CPFlowsheets\default.txaPackage'}

uses
  Forms,
  Classes,
  oCP_Exceptions,
  oInterfaces,
  uCP_GUI,
  Controls,
  SysUtils,
  Dialogs,
  uCCOW,
  fMain in 'fMain.pas' {frmMain},
  mEditObservationBattery in 'mEditObservationBattery.pas' {fraEditObservationBattery: TFrame},
  mEditObservationValue in 'mEditObservationValue.pas' {fraEditObservationValue: TFrame},
  mLogView in 'mLogView.pas' {fraLogView: TFrame},
  mReportView in 'mReportView.pas' {fraReportView: TFrame},
  mGridView in 'mGridView.pas' {fraGridView: TFrame},
  uGlobalVars in 'uGlobalVars.pas',
  mHL7Monitor in 'mHL7Monitor.pas' {fraHL7MonitorView: TFrame},
  fUserOptions in 'fUserOptions.pas' {frmUserOptions},
  fAlarmSettings in 'fAlarmSettings.pas' {frmAlarmSettings},
  mAlarmView in 'mAlarmView.pas' {fraAlarmView: TFrame},
  mTaskView in 'mTaskView.pas' {fraTaskView: TFrame},
  fSelectNewPage in '..\Common\forms\fSelectNewPage.pas' {frmSelectNewPage},
  fPatientLookup in '..\Common\forms\fPatientLookup.pas' {frmPatientLookup},
  fRestrictedRecord in '..\Common\forms\fRestrictedRecord.pas' {frmRestrictedRecord},
  fNoteWriter in '..\Common\forms\fNoteWriter.pas' {frmNoteWriter},
  mFlowsheetTotals in 'mFlowsheetTotals.pas' {fraFlowsheetTotals: TFrame},
  fSelectDateRange in '..\Common\forms\fSelectDateRange.pas' {frmSelectDateRange},
  fObservationViewer in '..\Common\forms\fObservationViewer.pas' {frmObservationViewer},
  fKardexOrder in 'fKardexOrder.pas' {frmAddEditKardexItem},
  fKardexCustomTimes in 'fKardexCustomTimes.pas' {frmKardexCustomTimes},
  fKardexViewer in '..\Common\forms\fKardexViewer.pas' {frmKardexViewer},
  fKardexHold in 'fKardexHold.pas' {frmKardexHold},
  fKardexAudits in 'fKardexAudits.pas' {frmKardexAudits},
  fKardexAddEvent in 'fKardexAddEvent.pas' {frmKardexAddEvent},
  fSelectDate in '..\Common\forms\fSelectDate.pas' {frmSelectDate},
  u508Dialogs in '..\Common\utilities\u508Dialogs.pas' {frmMessageDlg508},
  fSelectShift in '..\Common\forms\fSelectShift.pas' {frmSelectShift},
  fSelectDateTime in '..\Common\forms\fSelectDateTime.pas' {frmSelectDateTime},
  fAboutDlg in '..\Common\forms\fAboutDlg.pas' {frmAboutDlg},
  fSplash in '..\Common\forms\fSplash.pas' {frmSplash},
  fComment in '..\Common\forms\fComment.pas' {frmGetComment},
  fTimeOutManager in '..\Common\forms\fTimeOutManager.pas' {frmTimeOutManager},
  fImageList in 'fImageList.pas' {frmImageList};

{$R *.res}
//var
//  TimeOffset: integer;

const
  NOCCOW = 'noccow';

begin
  Application.Initialize;
  Application.Title := 'CP Flowsheets';
  if FindCmdLineSwitch('noccow', ['/', '-'], True) then
    begin
      MyVistALogin := NewDBConnection_VistANoCCOW;
      MyCCOWContextor := nil;
    end
  else
    begin
      MyVistALogin := NewDBConnection_VistA;
      if MyVistALogin.QueryInterface(ICCOWContextor, MyCCOWContextor) = 0 then
        if MyCCOWContextor.CCOWLinkStatus = lsNoCCOW then
          MyCCOWContextor := nil;
    end;

  try
    MyVistALogin.Connect([GetCommandLineValue('SERVER'), GetCommandLineValue('PORT'), '', 'MD CLIO']);
    MyUser := NewUser('', MyVistALogin);
    MyCPParameters := NewCPParameters(MyVistALogin);
    MyCPParameters.Load;


    if not MyCPParameters.getApplicationRegistered then
      MessageDlg508(
        Format('Unable to start %s.' + #13 + #13 +
        'This version ''%s'' is not the correct version.' + #13 + #13 +
        'Your VistA connection to %s shows' + #13 +
        '''%s'' as the required version.',
        [Application.ExeName, MyCPParameters.getCurrentApplicationVersion, MyVistALogin.getDescription, MyCPParameters.getRegisteredApplicationVersion]),
        mtError, [mbOk], 0)
{$IFDEF DEBUG}
      ;
      if MessageDlg508('DEBUG build ignores the version control' + #13#13 +
        'DO NOT USE IT IN PRODUCTION ACCOUNTS',
        mtInformation, [mbOK, mbCancel],0) = mrOK then
{$ELSE}
    else
{$ENDIF}
      try
        (*

        TimeOffset := GetServerTimeOffset(MyVistALogin);

        case TimeOffset of
          -1: raise ECPException.Create('Unable to verify server date/time. Application is unable to proceed.');
          0..120: {within two minutes...who cares};
          121..300: MessageDlg508('Workstation clock is off by '+IntToStr(TimeOffset)+' seconds.',mtWarning,[mbOk],0);
        else
          raise ECPException.CreateFmt('Workstation clock must be within 300 seconds of the server but it is off by %d seconds.',[TimeOffset]);
        end;
        *)
        
        frmImageList := TfrmImageList.Create(nil);
        try
          Application.CreateForm(TfrmMain, frmMain);
          Application.HintHidePause := 10000;
          Application.Run;
        finally
          FreeAndNil(frmImageList);
        end;
      except
        ShowMessage('This application has exited abnormally.');
      end;
  except
    on E: ECPException do
      begin
        ShowMessage('Application is shutting down.' + #13 + #13 + 'Reason:' + #13 + E.Message);
      end
  else
    raise;
  end;

  MyUser := nil;

end.

