unit fAlarmSettings;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  oCliO,
  oCliOTypes,
  ActnList,
  CheckLst, VA508AccessibilityManager;

type
  TfrmAlarmSettings = class(TForm)
    pnlButtons: TPanel;
    btn03: TButton;
    btn02: TButton;
    btn01: TButton;
    ActionList1: TActionList;
    acApply: TAction;
    acOK: TAction;
    acSetTerm: TAction;
    acResetForm: TAction;
    acChangeType: TAction;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    edtRangeBound1: TEdit;
    edtRangeBound2: TEdit;
    cbxTerm: TComboBox;
    cbxUnit: TComboBox;
    edtActivationComment: TEdit;
    lblHeader: TLabel;
    Shape1: TShape;
    acChangeMade: TAction;
    lblInRange: TLabel;
    cklbxValues: TCheckListBox;
    Label4: TLabel;
    v508AccessMgr: TVA508AccessibilityManager;
    cbxCondition: TComboBox;
    procedure acSetTermExecute(Sender: TObject);
    procedure acResetFormExecute(Sender: TObject);
    procedure acApplyExecute(Sender: TObject);
    procedure acOKExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure acChangeMadeExecute(Sender: TObject);
    procedure cbxConditionSelect(Sender: TObject);
    procedure acChangeTypeExecute(Sender: TObject);
  private
    fCliOAlarm: ICliOAlarm;
    { Private declarations }
  public
    { Public declarations }
  end;

procedure EditPatientAlarm(aCliOAlarm: ICliOAlarm);

implementation

uses
  uGlobalVars,
  oInterfaces,
  u508Dialogs;

{$R *.dfm}

const
  ALARM_TYPES: Array [TClioAlarmType] of string = ('Value BETWEEN', 'Value NOT BETWEEN',
    'Value GREATER THAN', 'Value LESS THAN', 'Value IN LIST', 'Value EQUALS',
    'Value does NOT EQUAL');

procedure EditPatientAlarm(aCliOAlarm: ICliOAlarm);
var
  i: integer;
  aTerm: ITerm;
begin
  with TfrmAlarmSettings.Create(Application) do
    try
      if Assigned(aCliOAlarm) then
        fCliOAlarm := aCliOAlarm
      else
        fCliOAlarm := MyAlarmList.getNewAlarm;

      MyTerminology.LoadList(cbxTerm.Items, ttObservation);
      for i := cbxTerm.Items.Count - 1 downto 0 do
        if cbxTerm.Items.Objects[i].GetInterface(ITerm, aTerm) then
          if not (aTerm.getDataType in [dtNumeric, dtBoolean, dtPickList]) then
            cbxTerm.Items.Delete(i);
      acResetForm.Execute;
      ShowModal;
    finally
      Free;
    end;
end;

function ALARM_TYPEToTClioAlarmType(const AlarmType: string): TCliOAlarmType;
begin
  Result := atBooleanFalse;
  {(atWithinRange, atNotWithinRange, atGreaterThan, atLessThan, atEqualsPickList, atBooleanTrue, atBooleanFalse)}
  if AlarmType = ALARM_TYPES[atWithinRange] then
    result := atWithinRange;
  if AlarmType = ALARM_TYPES[atNotWithinRange] then
    result := atNotWithinRange;
  if AlarmType = ALARM_TYPES[atGreaterThan] then
    result := atGreaterThan;
  if AlarmType = ALARM_TYPES[atLessThan] then
    result := atLessThan;
  if AlarmType = ALARM_TYPES[atEqualsPickList] then
    result := atEqualsPickList;
  if AlarmType = ALARM_TYPES[atBooleanTrue] then
    result := atBooleanTrue;
end;

procedure TfrmAlarmSettings.FormCreate(Sender: TObject);
begin
  lblHeader.Font.Style := [fsBold];
end;

procedure TfrmAlarmSettings.acApplyExecute(Sender: TObject);
var
  aTerm: ITerm;
  aUnit: ITerm;
  aValue: ITerm;
  aValueList: string;
//  aType: TCliOAlarmType;
  aMin: Double;
  aMax: Double;
  i: integer;
begin
  try
    if cbxTerm.ItemIndex < 0 then
      raise Exception.Create('No term selected.');

    aTerm := nil;
    aUnit := nil;
    aValue := nil;
    aMin := 0;
    aMax := 0;


    if cbxTerm.Items.Objects[cbxTerm.ItemIndex].GetInterface(ITerm, aTerm) then
    begin
      case aTerm.getDataType of
        dtNumeric:
          begin
            if cbxUnit.ItemIndex < 0 then
              raise Exception.Create('Unit must be selected for numeric values.');

            if not cbxUnit.Items.Objects[cbxUnit.ItemIndex].GetInterface(ITerm, aUnit) then
              raise Exception.Create('Error accessing unit.');
          end;

        dtPickList: ;
        else
          raise Exception.Create('Invalid term type for an alarm.' + #13 + 'Alarms can only be set for numeric or picklist items.');
        end;

        case fClioAlarm.getAlarmType of
          atWithinRange:
            begin
              aMin := StrToFloat(edtRangeBound1.Text);
              aMax := StrToFloat(edtRangeBound2.Text);
              if aMax < aMin then
                raise Exception.Create('Maximum must be greater or equal to minimum for valid ranges.')
            end;
          atNotWithinRange:
            begin
              aMin := StrToFloat(edtRangeBound1.Text);
              aMax := StrToFloat(edtRangeBound2.Text);
              if aMax < aMin then
                raise Exception.Create('Maximum must be greater or equal to minimum for valid ranges.')
            end;
          atGreaterThan:
            begin
              aMin := 0;
              aMax := StrToFloat(edtRangeBound1.Text);
            end;
          atLessThan:
            begin
              aMin := StrToFloat(edtRangeBound1.Text);
              aMax := 0;
            end;
          atEqualsPickList:
            begin
              aMin := 0;
              aMax := 0;
              aValueList := '';
              with TStringList.Create do
                try
                  for i := 0 to cklbxValues.Items.Count - 1 do
                    if cklbxValues.Checked[i] then
                      if cklbxValues.Items.Objects[i].GetInterface(ITerm, aValue) then
                        Add(aValue.getID);
                  aValueList := CommaText;
                finally
                  Free;
                end;
              if aValueList = '' then
                raise Exception.Create('No picklist items selected.');
            end;
          atBooleanTrue:
            begin
            end;
          atBooleanFalse:
            begin
            end;
          else
            raise Exception.Create('Invalid alarm type for this term.');
          end;
      end
    else
      raise Exception.Create('Error accessing term.');

    if not Assigned(fCliOAlarm) then
      fCliOAlarm := MyAlarmList.getNewAlarm;

    with fCliOAlarm do
      begin
        setTerm(aTerm);
        setUnit(aUnit);
        setMinValue(aMin);
        setMaxValue(aMax);
        setPickListValues(aValueList);
        setActivationComment(edtActivationComment.Text);
        Update(MyUser);
      end;
    acApply.Enabled := False;
  except
    on E: EConvertError do
      MessageDlg508('Error saving alarm, non-numeric value(s) have been entered.', mtError, [mbOk], 0);
    on E: Exception do
      MessageDlg508('Error saving alarm.' + #13 + #13 + 'Reason: ' + E.Message, mtError, [mbOk], 0);
  end;
end;

procedure TfrmAlarmSettings.acChangeMadeExecute(Sender: TObject);
begin
  acApply.Enabled := True;
end;

procedure TfrmAlarmSettings.acChangeTypeExecute(Sender: TObject);
var
  Item: TVA508AccessibilityItem;
begin
  //First, we hide everything
  edtRangeBound1.Visible := false;
  lblInRange.Visible := false;
  edtRangeBound2.Visible := false;
  cklbxValues.Visible := false;

  case fCliOAlarm.getAlarmType of
  atWithinRange:
    begin
      edtRangeBound1.Enabled := true;
      edtRangeBound1.Visible := true;
      edtRangeBound1.ReadOnly := false;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := edtRangeBound1;
      Item.AccessText := ALARM_TYPES[atWithinRange] + ' low bound';
      lblInRange.Visible := true;
      edtRangeBound2.Enabled := true;
      edtRangeBound2.Visible := true;
      edtRangeBound2.ReadOnly := false;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := edtRangeBound2;
      Item.AccessText := ALARM_TYPES[atWithinRange] + ' high bound';
    end;
  atNotWithinRange:
    begin
      edtRangeBound1.Enabled := true;
      edtRangeBound1.Visible := true;
      edtRangeBound1.ReadOnly := false;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := edtRangeBound1;
      Item.AccessText := ALARM_TYPES[atNotWithinRange] + ' low bound';
      lblInRange.Visible := true;
      edtRangeBound2.Enabled := true;
      edtRangeBound2.Visible := true;
      edtRangeBound2.ReadOnly := false;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := edtRangeBound2;
      Item.AccessText := ALARM_TYPES[atNotWithinRange] + ' high bound';
    end;
  atGreaterThan:
    begin
      edtRangeBound1.Enabled := true;
      edtRangeBound1.Visible := true;
      edtRangeBound1.ReadOnly := false;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := edtRangeBound1;
      Item.AccessText := ALARM_TYPES[atGreaterThan];
    end;
  atLessThan:
    begin
      edtRangeBound1.Enabled := true;
      edtRangeBound1.Visible := true;
      edtRangeBound1.ReadOnly := false;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := edtRangeBound1;
      Item.AccessText := ALARM_TYPES[atLessThan];
    end;
  atEqualsPickList:
    begin
      cklbxValues.Enabled := true;
      cklbxValues.Visible := true;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := cklbxValues;
      Item.AccessText := ALARM_TYPES[atEqualsPickList];
    end;
  atBooleanTrue:
    begin
      edtRangeBound1.Visible := true;
      edtRangeBound1.ReadOnly := true;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := cklbxValues;
      Item.AccessText := ALARM_TYPES[atBooleanTrue];
    end;
  atBooleanFalse:
    begin
      edtRangeBound1.Visible := true;
      edtRangeBound1.ReadOnly := true;
      Item := v508AccessMgr.AccessData.Add;
      Item.Component := cklbxValues;
      Item.AccessText := ALARM_TYPES[atBooleanFalse];
    end;
  end;

  v508AccessMgr.RefreshComponents;
end;

procedure TfrmAlarmSettings.acOKExecute(Sender: TObject);
begin
  acApply.Execute;
  if not acApply.Enabled then
    ModalResult := mrOk;
end;

procedure TfrmAlarmSettings.acResetFormExecute(Sender: TObject);
var
  i: integer;
  aValue: ITerm;
begin
  cbxCondition.Items.Clear;
  edtRangeBound1.Visible := false;
  lblInRange.Visible := false;
  edtRangeBound2.Visible := false;
  cklbxValues.Visible := false;

  if Assigned(fCliOAlarm) then
    begin
      if fCliOAlarm.getTerm <> nil then
        cbxTerm.ItemIndex := MyTerminology.GetTermIndex(cbxTerm.Items, fCliOAlarm.getTerm.getID)
      else
        cbxTerm.ItemIndex := -1;

      acSetTerm.Execute;

      if fCliOAlarm.getUnit <> nil then
        cbxUnit.ItemIndex := MyTerminology.GetTermIndex(cbxUnit.Items, fCliOAlarm.getUnit.getID)
      else
        cbxUnit.ItemIndex := -1;
      edtRangeBound1.ReadOnly := false;
      case fCliOAlarm.getAlarmType of
        atLessThan:
          begin
            cbxCondition.ItemIndex := cbxCondition.Items.IndexOf(ALARM_TYPES[atLessThan]);
            edtRangeBound1.Text := FloatToStr(fCliOAlarm.getMinValue);
          end;
        atGreaterThan:
          begin
            cbxCondition.ItemIndex := cbxCondition.Items.IndexOf(ALARM_TYPES[atGreaterThan]);
            edtRangeBound1.Text := FloatToStr(fCliOAlarm.getMaxValue);
          end;
        atWithinRange:
          begin
            cbxCondition.ItemIndex := cbxCondition.Items.IndexOf(ALARM_TYPES[atWithinRange]);
            edtRangeBound1.Text := FloatToStr(fCliOAlarm.getMinValue);
            edtRangeBound2.Text := FloatToStr(fCliOAlarm.getMaxValue);
          end;
        atNotWithinRange:
          begin
            cbxCondition.ItemIndex := cbxCondition.Items.IndexOf(ALARM_TYPES[atNotWithinRange]);
            edtRangeBound1.Text := FloatToStr(fCliOAlarm.getMinValue);
            edtRangeBound2.Text := FloatToStr(fCliOAlarm.getMaxValue);
          end;
        atEqualsPickList:
          begin
            cbxCondition.ItemIndex := cbxCondition.Items.IndexOf(ALARM_TYPES[atEqualsPickList]);
            for i := 0 to cklbxValues.Items.Count - 1 do
              if cklbxValues.Items.Objects[i].GetInterface(ITerm, aValue) then
                cklbxValues.Checked[i] := (Pos(aValue.getID, fCliOAlarm.getPickListValues) > 0);
            cklbxValues.Visible := true;
            edtRangeBound1.Visible := false;
            edtRangeBound2.Visible := false;
            lblInRange.Visible := false;
          end;
        atBooleanTrue:
          begin
            cbxCondition.ItemIndex := cbxCondition.Items.IndexOf(ALARM_TYPES[atBooleanTrue]);
            edtRangeBound1.Text := fCliOAlarm.getTerm.getBooleanValueTrue;
            edtRangeBound1.ReadOnly := true;
          end;
        atBooleanFalse:
          begin
            cbxCondition.ItemIndex := cbxCondition.Items.IndexOf(ALARM_TYPES[atBooleanFalse]);
            edtRangeBound1.Text := fCliOAlarm.getTerm.getBooleanValueTrue;
            edtRangeBound1.ReadOnly := true;
          end;
      end;
      acChangeType.Execute;
      edtActivationComment.Text := fCliOAlarm.getActivationComment;
    end;
  acApply.Enabled := False;
end;

procedure TfrmAlarmSettings.acSetTermExecute(Sender: TObject);
var
  aTerm: ITerm;
begin
  if cbxTerm.ItemIndex > -1 then
    if cbxTerm.Items.Objects[cbxTerm.ItemIndex].GetInterface(ITerm, aTerm) then
      begin
        aTerm.LoadListOfPairs(ttUnit, cbxUnit.Items);
        cbxUnit.ItemIndex := -1;
        cbxUnit.Enabled := (cbxUnit.Items.Count > 0);
        if cbxUnit.Items.Count = 1 then
          cbxUnit.ItemIndex := 0;

        cbxCondition.Items.Clear;
        cklbxValues.Items.Clear;

        edtRangeBound1.ReadOnly := false;
        case aTerm.getDataType of
        dtNumeric:
          begin
            cbxCondition.Items.Add(ALARM_TYPES[atWithinRange]);
            cbxCondition.Items.Add(ALARM_TYPES[atNotWithinRange]);
            cbxCondition.Items.Add(ALARM_TYPES[atGreaterThan]);
            cbxCondition.Items.Add(ALARM_TYPES[atLessThan]);
            edtRangeBound1.Visible := true;
          end;
        dtBoolean:
          begin
            cbxCondition.Items.Add(ALARM_TYPES[atBooleanTrue]);
            cbxCondition.Items.Add(ALARM_TYPES[atBooleanFalse]);
            edtRangeBound1.Text := aTerm.getBooleanValueTrue;
            edtRangeBound1.ReadOnly := True;
          end;
        dtPickList:
          begin
            cbxCondition.Items.Add(ALARM_TYPES[atEqualsPickList]);
            cbxCondition.ItemIndex := 0;
            aTerm.LoadListOfPairs(ttValue, cklbxValues.Items);
            cklbxValues.Visible := true;
            edtRangeBound1.Visible := false;
            edtRangeBound2.Visible := false;
            cklbxValues.Enabled := (cklbxValues.Items.Count > 0);
            cbxConditionSelect(cbxCondition);
          end;
        else
          Exit;
        end;
      end;
  acChangeMade.Execute;
end;

procedure TfrmAlarmSettings.cbxConditionSelect(Sender: TObject);
begin
  fCliOAlarm.setAlarmType(ALARM_TYPEToTClioAlarmType(cbxCondition.Text));
  acChangeType.Execute;
end;

end.

