unit fKardexOrder;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  CheckLst,
  ComCtrls,
  oCPTypes,
  oCliOTypes,
  oInterfaces,
  uGlobalVars,
  ExtCtrls,
  ActnList,
  Buttons, VA508AccessibilityManager, System.Actions;

type
  TfrmAddEditKardexItem = class(TForm)
    pgctrl: TPageControl;
    tbshtContinuous: TTabSheet;
    tbshtMain: TTabSheet;
    Label1: TLabel;
    Label3: TLabel;
    Label2: TLabel;
    cbxCareAction: TComboBox;
    dtpStartDate: TDateTimePicker;
    dtpStartTime: TDateTimePicker;
    edtDuration: TEdit;
    udDuration: TUpDown;
    rgScheduleType: TRadioGroup;
    Label6: TLabel;
    tbshtCustomTimes: TTabSheet;
    Label4: TLabel;
    cbxSchedule: TComboBox;
    cbxFirstTime: TComboBox;
    Label7: TLabel;
    tbshtDisplay: TTabSheet;
    Panel1: TPanel;
    btnCancel: TButton;
    btnNext: TButton;
    btnBack: TButton;
    gbxPRNOption: TGroupBox;
    rbtnNoPRNOption: TRadioButton;
    rbtnAndPRN: TRadioButton;
    rbtnAndPRNNTE: TRadioButton;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    ActionList1: TActionList;
    acNext: TAction;
    acBack: TAction;
    acCancel: TAction;
    acFinish: TAction;
    rbtnOrPRN: TRadioButton;
    Label11: TLabel;
    acOrderDisplay: TAction;
    Label12: TLabel;
    edtPRNOrderLimit: TEdit;
    udPRNOrderLimit: TUpDown;
    chkbxPRNOrderLimit: TCheckBox;
    lblPRNOrderLimit: TLabel;
    GroupBox1: TGroupBox;
    lblScheduleTimes: TLabel;
    gbxDetails: TGroupBox;
    gbxFirstTime: TGroupBox;
    chkbxDone: TCheckBox;
    lblDateTimeCompleted: TLabel;
    dtpCompletionDate: TDateTimePicker;
    dtpCompletionTime: TDateTimePicker;
    lblCompletionComment: TLabel;
    acOrderBuild: TAction;
    edtPRNNTE: TEdit;
    udPRNNTE: TUpDown;
    lblPRNNTE: TLabel;
    acPRNOptionChange: TAction;
    memSpecialInstructions: TMemo;
    memCompletionComment: TMemo;
    acChangeSchedule: TAction;
    acPRNOrderLimitChange: TAction;
    acChangeScheduleType: TAction;
    btnChangeSchedule: TButton;
    acBuildCustomSchedule: TAction;
    acSetFirstTime: TAction;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure FormCreate(Sender: TObject);
    procedure acBackExecute(Sender: TObject);
    procedure acNextExecute(Sender: TObject);
    procedure acOrderDisplayExecute(Sender: TObject);
    procedure acCancelExecute(Sender: TObject);
    procedure acFinishExecute(Sender: TObject);
    procedure chkbxDoneClick(Sender: TObject);
    procedure acOrderBuildExecute(Sender: TObject);
    procedure acPRNOptionChangeExecute(Sender: TObject);
    procedure acChangeScheduleExecute(Sender: TObject);
    procedure acPRNOrderLimitChangeExecute(Sender: TObject);
    procedure acChangeScheduleTypeExecute(Sender: TObject);
    procedure acBuildCustomScheduleExecute(Sender: TObject);
    procedure acSetFirstTimeExecute(Sender: TObject);
    procedure cbxFirstTimeChange(Sender: TObject);
  private
    fOrderErrors: TStringList;
    fKardexAction: ICPKardexAction;
    fAction: ITerm;
    fSchedule: ICPSchedule;
    fScheduleType: TCPScheduleType;
    fScheduleTypePRN: TCPScheduleTypePRN;
    fPRNOrderLimit: integer;
    fPRNTimes: integer;
    fStartDateTime: TDateTime;
    fStopDateTime: TDateTime;
    fDuration: integer;
    fSpecialInstructions: string;
    fCustomSchedule: string;
  public
    { Public declarations }
  end;

var
  frmAddEditKardexItem: TfrmAddEditKardexItem;

implementation

{$R *.dfm}

uses
  DateUtils,
  fKardexCustomTimes,
  u508Dialogs;

const
  DEFAULT_INTERVAL = 30;

procedure TfrmAddEditKardexItem.FormCreate(Sender: TObject);
var
  aMinute: integer;
  aType: TCPScheduleType;
begin
  fOrderErrors := TStringList.Create;
  fKardexAction := MyCPKardex.getNewAction(MyPatient);
  tbshtContinuous.TabVisible := False;
  tbshtMain.TabVisible := False;
  tbshtCustomTimes.TabVisible := False;
  tbshtDisplay.TabVisible := False;
  pgctrl.ActivePage := tbshtMain;
  cbxFirstTime.Items.Clear;

  rgScheduleType.Items.Clear;
  for aType := Low(TCPScheduleType) to High(TCPScheduleType) do
    rgScheduleType.Items.Add(CP_SCHEDULE_TYPE[aType]);
  rgScheduleType.ItemIndex := Ord(stContinuous);

  aMinute := 0;
  while aMinute < 1440 do
    begin
      cbxFirstTime.Items.Add(FormatDateTime('hhmm', IncMinute(0, aMinute)));
      Inc(aMinute, DEFAULT_INTERVAL);
    end;

  dtpStartDate.Date := Now;
  dtpStartTime.Time := Now;
  dtpCompletionDate.Date := Now;
  dtpCompletionTime.Time := IncMinute(Now, 1);
end;

procedure TfrmAddEditKardexItem.acNextExecute(Sender: TObject);
begin
  acOrderBuild.Execute;
  case pgctrl.ActivePageIndex of
    0: {Main, check for non-continuous}
      begin
        if cbxCareAction.ItemIndex < 0 then
          MessageDlg508('Please select a care action', mtInformation, [mbok], 0)
        else if rgScheduleType.ItemIndex = 0 then
          pgctrl.ActivePageIndex := pgctrl.ActivePageIndex + 1
        else
          begin
            acOrderDisplay.Execute;
            btnNext.Action := acFinish;
            pgctrl.ActivePage := tbshtDisplay;
          end
      end;
    1: {Continuous}
      begin
        if (cbxSchedule.ItemIndex > -1) and (cbxFirstTime.ItemIndex > -1) then
          begin
            acOrderDisplay.Execute;
            btnNext.Action := acFinish;
            pgctrl.ActivePage := tbshtDisplay;
          end
        else
          MessageDlg508('Missing schedule settings', mtInformation, [mbOk], 0);
      end;
    2: {Continuous - Custom Times}
      begin
        acOrderDisplay.Execute;
        btnNext.Action := acFinish;
        pgctrl.ActivePage := tbshtDisplay;
      end;
  end;
  acBack.Enabled := True;
end;

procedure TfrmAddEditKardexItem.acBackExecute(Sender: TObject);
//var
//  i: integer;
begin
  case pgctrl.ActivePageIndex of
    0: MessageDlg508('You are at the beginning of this Wizard.', mtInformation, [mbOk], 0); {Shouldn't happen but who knows}
    1..2: pgctrl.ActivePageIndex := pgctrl.ActivePageIndex - 1;
    3:
      case rgScheduleType.ItemIndex of
        0: pgctrl.ActivePage := tbshtContinuous;
      else
        pgctrl.ActivePage := tbshtMain;
      end;
  end;
  btnNext.Action := acNext;
  acBack.Enabled := (pgctrl.ActivePageIndex > 0);
end;

procedure TfrmAddEditKardexItem.acOrderBuildExecute(Sender: TObject);
begin
  fOrderErrors.Clear;

  fStartDateTime := trunc(dtpStartDate.Date) + Frac(dtpStartTime.Time);
  fDuration := udDuration.Position;
  fStopDateTime := trunc(fStartDateTime) + fDuration + EncodeTime(23, 59, 59, 0);
  fSpecialInstructions := memSpecialInstructions.Text;

  if cbxCareAction.ItemIndex > -1 then
    cbxCareAction.Items.Objects[cbxCareAction.ItemIndex].getInterface(ITerm, fAction)
  else
    begin
      fAction := nil;
      fOrderErrors.Add('No care action selected.');
      Exit;
    end;

  if rgScheduleType.ItemIndex = -1 then
    begin
      fOrderErrors.Add('Type of scheduled must be selected to continue.');
      Exit;
    end
  else
    fScheduleType := TCPScheduleType(rgScheduleType.ItemIndex);

  case fScheduleType of
    stContinuous:
      begin
        fSchedule := nil;
        fScheduleTypePRN := prnNot;

        if cbxSchedule.ItemIndex > -1 then
          begin
            cbxSchedule.Items.Objects[cbxSchedule.ItemIndex].getInterface(ICPSchedule, fSchedule);

            if rbtnNoPRNOption.Checked then
              fScheduleTypePRN := prnNot
            else if rbtnAndPRN.Checked then
              fScheduleTypePRN := prnAnd
            else if rbtnAndPRNNTE.Checked then
              fScheduleTypePRN := prnAndNTE
            else if rbtnOrPRN.Checked then
              fScheduleTypePRN := prnOr
            else
              fScheduleTypePRN := prnNot;

            if fScheduleTypePRN = prnAndNTE then
              fPRNTimes := udPRNNTE.Position
            else
              fPRNTimes := 0;
          end
        else
          begin
            fSchedule := nil;
            fOrderErrors.Add('No Schedule selected.');
            Exit;
          end;
      end;
    stPRN:
      begin
        if chkbxPRNOrderLimit.Checked then
          fPRNOrderLimit := udPRNOrderLimit.Position
        else
          fPRNOrderLimit := 0;
      end;
  end;

  with fKardexAction do
    begin
      setTask(fAction);
      setSchedule(fSchedule);
      setStartDateTime(fStartDateTime);
      setStopDateTime(fStopDateTime);
      setScheduleType(fScheduleType);
      setSchedulePRNType(fScheduleTypePRN);
      setStatus(asActive);
      setSpecialInstructions(fSpecialInstructions);
      setPRNNTE(fPRNTimes);
      setScheduleTimes(fCustomSchedule);
    end;
end;

procedure TfrmAddEditKardexItem.acOrderDisplayExecute(Sender: TObject);
var
  i: integer;
  aScheduleDisplay: string;
//  aScheduleTimes: TStringList;

  procedure AddLine(aPrompt, aValue: string);
  var
    aLabel: TLabel;
    aLeft: integer;
    aTop: integer;
  begin
    aTop := (trunc(gbxDetails.ComponentCount / 2) * 25) + 25;
    aLeft := 8;
    aLabel := TLabel.Create(gbxDetails);
    with aLabel do
      begin
        Parent := gbxDetails;
        Left := aLeft;
        Top := aTop;
        Caption := aPrompt;
        Font.Style := [fsBold];
        aLeft := aLeft + Width + 10;
      end;

    aLabel := TLabel.Create(gbxDetails);
    with aLabel do
      begin
        Parent := gbxDetails;
        Left := aLeft;
        Top := aTop;
        WordWrap := True;
        Caption := aValue;
        Width := gbxDetails.Width - aLeft - 10;
        Font.Style := [];
      end;
  end;

begin
  acOrderBuild.Execute;

  acFinish.Enabled := (fOrderErrors.Count = 0);

  while gbxDetails.ComponentCount > 0 do
    gbxDetails.Components[0].Free;

  if fOrderErrors.Count > 0 then
    begin
      AddLine('Notice:', 'Errors found in order.');
      for i := 0 to fOrderErrors.Count - 1 do
        AddLine(Format('%2d:', [i]), fOrderErrors[i]);
      acFinish.Enabled := False;
      Exit;
    end;

  AddLine('Item:',
    Format('%s (%s)', [fKardexAction.getCareAction.getDisplayName, fKardexAction.getCareAction.getTerm]));

  AddLine('Start Date/Time:',
    FormatDateTime('MM/DD/YYYY hh:mm', fKardexAction.getStartDateTime));

  AddLine('Duration (Days):',
    Format('%d', [fKardexAction.getDuration]));

  AddLine('Stop Date/Time:',
    FormatDateTime('MM/DD/YYYY hh:mm', fKardexAction.getStopDateTime));

  aScheduleDisplay := CP_SCHEDULE_TYPE[TCPScheduleType(fKardexAction.getScheduleType)];

  if fScheduleType = stContinuous then
    begin
      aScheduleDisplay := aScheduleDisplay + ' (' + fKardexAction.getSchedule.getName; //fSchedule.getName;

      if fKardexAction.getSchedulePRNType <> prnNot then
        aScheduleDisplay := aScheduleDisplay + ' ' +
          CP_SCHEDULE_TYPE_PRN[fKardexAction.getSchedulePRNType];

      if fKardexAction.getSchedulePRNType = prnAndNTE then
        aScheduleDisplay := aScheduleDisplay + ' ' +
          Format('%d time(s)', [fKardexAction.getPRNNTE]);

      aScheduleDisplay := aScheduleDisplay + ')';
    end;

  if fKardexAction.getScheduleType = stPRN then
    if fPRNOrderLimit > 0 then
      aScheduleDisplay := Format('%s Limit to %d times', [CP_SCHEDULE_TYPE[fKardexAction.getScheduleType], fKardexAction.getPRNNTE])
    else
      aScheduleDisplay := CP_SCHEDULE_TYPE[fKardexAction.getScheduleType];

  AddLine('Schedule:', aScheduleDisplay);

  AddLine('Special Instructions:', fKardexAction.getSpecialInstructions);

  fKardexAction.BuildSchedule;

  if fKardexAction.getFirstEvent <> nil then
    chkbxDone.Caption := Format('I''ve already completed the event for %s.', [FormatDateTime('YYYY-MM-DD hh:mm', fKardexAction.getFirstEvent.getScheduledDateTime)]);
end;

procedure TfrmAddEditKardexItem.acPRNOptionChangeExecute(Sender: TObject);
begin
  edtPRNNTE.Enabled := rbtnAndPRNNTE.Checked;
  udPRNNTE.Enabled := rbtnAndPRNNTE.Checked;
  lblPRNNTE.Enabled := rbtnAndPRNNTE.Checked;
end;

procedure TfrmAddEditKardexItem.acPRNOrderLimitChangeExecute(Sender: TObject);
begin
  edtPRNOrderLimit.Enabled := (rgScheduleType.ItemIndex = 1) and (chkbxPRNOrderLimit.Checked);
  udPRNOrderLimit.Enabled := (rgScheduleType.ItemIndex = 1) and (chkbxPRNOrderLimit.Checked);
  lblPRNOrderLimit.Enabled := (rgScheduleType.ItemIndex = 1) and (chkbxPRNOrderLimit.Checked);
  udPRNOrderLimit.Invalidate; {ud widget doesn't redraw consistently}
end;

procedure TfrmAddEditKardexItem.acSetFirstTimeExecute(Sender: TObject);
var
  i: integer;
  H, M: Word;
  aDateTime: TDateTime;
begin
  for i := 0 to cbxFirstTime.Items.Count - 1 do
    begin
      H := StrToInt(Copy(cbxFirstTime.Items[i], 1, 2));
      M := StrToInt(Copy(cbxFirstTime.Items[i], 3, 2));
      aDateTime := trunc(fStartDateTime) + EncodeTime(H, M, 0, 0);
      if aDateTime >= fStartDateTime then
        begin
          cbxFirstTime.ItemIndex := i;
          Break;
        end;
    end;
end;

procedure TfrmAddEditKardexItem.acBuildCustomScheduleExecute(Sender: TObject);
var
  aTimes: string;
begin
  aTimes := lblScheduleTimes.Caption;
  if BuildCustomTimes(Self, aTimes) then
    begin
      fCustomSchedule := aTimes;
      lblScheduleTimes.Caption := aTimes;
      cbxFirstTime.Items.CommaText := aTimes;
      acSetFirstTime.Execute;
    end;
end;

procedure TfrmAddEditKardexItem.acCancelExecute(Sender: TObject);
begin
  if MessageDlg508('Cancel this Kardex order?', mtConfirmation, mbYesNo, 0) = mrYes then
    ModalResult := mrCancel;
end;

procedure TfrmAddEditKardexItem.acChangeScheduleExecute(Sender: TObject);
begin
  acBuildCustomSchedule.Enabled := False;
  fCustomSchedule := '';
  if cbxSchedule.ItemIndex > -1 then
    if cbxSchedule.Items.Objects[cbxSchedule.ItemIndex].GetInterface(ICPSchedule, fSchedule) then
      begin
        cbxFirstTime.Items.CommaText := fSchedule.getSchedule;
        cbxFirstTime.ItemIndex := -1;
        cbxFirstTime.Enabled := (cbxFirstTime.Items.Count > 0);

        acSetFirstTime.Execute;

        if not fSchedule.getMultiDay then
          begin
            fCustomSchedule := fSchedule.getSchedule;
            lblScheduleTimes.Caption := fSchedule.getSchedule;
          end
        else
          begin
            fCustomSchedule := fSchedule.getSchedule;
            lblScheduleTimes.Caption := cbxFirstTime.Text;
          end;
        acOrderBuild.Execute;
        acBuildCustomSchedule.Enabled := (not fSchedule.getMultiDay);
      end
    else
      fSchedule := nil
  else
    fSchedule := nil;
end;

procedure TfrmAddEditKardexItem.acChangeScheduleTypeExecute(Sender: TObject);
begin
  chkbxPRNOrderLimit.Enabled := (rgScheduleType.ItemIndex = 1);
  udPRNOrderLimit.Enabled := (rgScheduleType.ItemIndex < 3);

  case rgScheduleType.ItemIndex of
    0: udDuration.Position := 14;
    1: udDuration.Position := 7;
    2: udDuration.Position := 1;
  else
    udDuration.Position := 0;
  end;
end;

procedure TfrmAddEditKardexItem.acFinishExecute(Sender: TObject);
var
//  aFirstEvent: ICPKardexEvent;
  aFirstDate: TDateTime;
begin
  try
    MyCPKardex.InsertAction(fKardexAction);
    if chkbxDone.Checked then
      begin
        aFirstDate := trunc(dtpCompletionDate.Date) + frac(dtpCompletionTime.Time);
        if fKardexAction.getFirstEvent <> nil then
          MyCPKardex.CompleteEvent(fKardexAction.getFirstEvent, MyUser, aFirstDate, memCompletionComment.Text)
        else
          MyCPKardex.CompleteEvent(fKardexAction, MyUser, aFirstDate, memCompletionComment.Text);
      end;
    ModalResult := mrOK;
  except
    on E: Exception do
      MessageDlg508('Unable to file Kardex Action.' + #13 + 'Message: ' + E.Message,
        mtError, [mbok], 0);
  end;
end;

procedure TfrmAddEditKardexItem.cbxFirstTimeChange(Sender: TObject);
begin
  if fSchedule.getMultiDay then
    begin
      fCustomSchedule := cbxFirstTime.Text;
      lblScheduleTimes.Caption := cbxFirstTime.Text;
    end;
end;

procedure TfrmAddEditKardexItem.chkbxDoneClick(Sender: TObject);
begin
  lblDateTimeCompleted.Enabled := chkbxDone.Checked;
  dtpCompletionDate.Enabled := chkbxDone.Checked;
  dtpCompletionTime.Enabled := chkbxDone.Checked;
  lblCompletionComment.Enabled := chkbxDone.Checked;
  memCompletionComment.Enabled := chkbxDone.Checked;
end;

end.

