unit fUserOptions;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  oInterfaces,
  StdCtrls,
  ExtCtrls,
  ComCtrls,
  ImgList,
  ActnList,
  fImageList, System.Actions;

type
  TfrmUserOptions = class(TForm)
    pgctrl: TPageControl;
    tbshtFlowsheetColors: TTabSheet;
    Label3: TLabel;
    gbxAttributes: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    ckbxBold: TCheckBox;
    ckbxItalics: TCheckBox;
    ckbxUnderline: TCheckBox;
    clrbxBackground: TColorBox;
    clrbxForeground: TColorBox;
    cbxElements: TComboBox;
    gbxSamples: TGroupBox;
    pnlSamples: TPanel;
    Panel1: TPanel;
    btnApply: TButton;
    btnCancel: TButton;
    btnOK: TButton;
    Panel2: TPanel;
    tvOptions: TTreeView;
    Shape1: TShape;
    tbshtGeneralOptions: TTabSheet;
    cbxFlowsheets: TComboBox;
    edtRefreshSeconds: TEdit;
    udRefreshSeconds: TUpDown;
    ckbxConfirmOnClose: TCheckBox;
    Label4: TLabel;
    lblRefreshSeconds: TLabel;
    ckbxRefreshEnabled: TCheckBox;
    lblGeneralHeader: TLabel;
    Shape2: TShape;
    lblFlowsheetHeader: TLabel;
    Shape3: TShape;
    tbshtIntro: TTabSheet;
    lblIntro: TLabel;
    Panel3: TPanel;
    lblHeader: TLabel;
    ActionList1: TActionList;
    acApply: TAction;
    acOK: TAction;
    acChangeMade: TAction;
    Shape4: TShape;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Bevel4: TBevel;
    Label5: TLabel;
    cbxLocations: TComboBox;
    ckbxUsePatientLocation: TCheckBox;
    rgSkin: TRadioGroup;
    procedure cbxElementsChange(Sender: TObject);
    procedure FontChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tvOptionsChange(Sender: TObject; Node: TTreeNode);
    procedure ckbxRefreshEnabledClick(Sender: TObject);
    procedure tvOptionsGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure acApplyExecute(Sender: TObject);
    procedure acOKExecute(Sender: TObject);
    procedure acChangeMadeExecute(Sender: TObject);
    procedure edtRefreshSecondsExit(Sender: TObject);
    procedure udRefreshSecondsClick(Sender: TObject; Button: TUDBtnType);
    procedure edtRefreshSecondsKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    fUser: IUser;
    fOptions: ICPUserOptions;
    fLoading: boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

function UpdateUserSettings(aUser: IUser; aOptions: ICPUserOptions): boolean;

implementation

uses
  uGlobalVars,
  oCPTypes,
  oCliOTypes
  , System.UITypes // 2FA
  ;

var
  Samples: array of TLabel;
//  UserOptions: TfrmUserOptions;

{$R *.dfm}

function UpdateUserSettings(aUser: IUser; aOptions: ICPUserOptions): boolean;
var
  i: integer;
  aCPFlowsheet: ICPFlowsheet;
begin
//  if not Assigned(UserOptions) then
//    Application.CreateForm(TfrmUserOptions,UserOptions);

  with TfrmUserOptions.Create(Application) do
//  with UserOptions do
    try
      fUser := aUser;
      fOptions := aOptions;
      fLoading := True;
      fOptions.Load;
      Caption := Format('CP Flowsheets Settings for ''%s''', [aUser.getESIGName]);

      {Page 1 - General User Options}
      MyCPService.LoadList(cbxFlowsheets.Items, ciFlowsheet);
      cbxFlowsheets.Items.Insert(0, '{None}');
      cbxFlowsheets.ItemIndex := 0;

      MyPatientSource.LoadLocations(cbxLocations.Items);
      cbxLocations.Items.Insert(0, '{None}');
      cbxFlowsheets.ItemIndex := 0;

      rgSkin.ItemIndex := Ord(aOptions.getLookAndFeel);

      {Gotta do a loop to set the default already selected}
      if fOptions.getDefaultFlowsheetID <> '' then
        for i := 1 to cbxFlowsheets.Items.Count - 1 do
          if cbxFlowsheets.Items.Objects[i].GetInterface(ICPFlowsheet, aCPFlowsheet) then
            if aCPFlowsheet.getID = fOptions.getDefaultFlowsheetID then
              begin
                cbxflowsheets.ItemIndex := i;
                Break;
              end;

      ckbxRefreshEnabled.Checked := fOptions.getAutoRefreshEnabled;
      udRefreshSeconds.Position := fOptions.getAutoRefreshInterval;
      udRefreshSeconds.Enabled := fOptions.getAutoRefreshEnabled;

      edtRefreshSeconds.Text := IntToStr(fOptions.getAutoRefreshInterval);
      edtRefreshSeconds.Enabled := fOptions.getAutoRefreshEnabled;

      lblRefreshSeconds.Enabled := fOptions.getAutoRefreshEnabled;

      ckbxConfirmOnClose.Checked := fOptions.getConfirmOnClose;
      ckbxUsePatientLocation.Checked := fOptions.getUsePatientLocation;

      if fOptions.getDefaultLocationID <> '' then
        cbxLocations.ItemIndex := MyPatientSource.GetLocationIndexByID(cbxLocations.Items, fOptions.getDefaultLocationID);

      {Page 2 - Flowsheet Colors}
      SetLength(Samples, Length(OBSERVATION_RANGE_ORDER));
      for i := Low(OBSERVATION_RANGE_ORDER) to High(OBSERVATION_RANGE_ORDER) do
        begin
          Samples[i] := TLabel.Create(pnlSamples);
          Samples[i].Parent := pnlSamples;
          Samples[i].Top := i * Samples[i].Height;
          Samples[i].Align := alTop;
          Samples[i].Font := fOptions.getFlowsheetItemFont(OBSERVATION_RANGE_ORDER[i]);
          Samples[i].Caption := OBSERVATION_RANGE[OBSERVATION_RANGE_ORDER[i]];
          Samples[i].ParentColor := False;
          Samples[i].Transparent := False;
          Samples[i].Color := fOptions.getFlowsheetItemColor(OBSERVATION_RANGE_ORDER[i]);
          cbxElements.Items.Add(OBSERVATION_RANGE[OBSERVATION_RANGE_ORDER[i]]);
        end;

      fLoading := False;
      cbxElements.ItemIndex := 0;
      cbxElementsChange(nil);
      acApply.Enabled := False;
      ShowModal;
    finally
      Free;
    end;
    Result := True;
end;

procedure TfrmUserOptions.acApplyExecute(Sender: TObject);
var
  i: integer;
  aFlowsheet: ICPFlowsheet;
  aLocation: ICPLocation;
begin
  fOptions.setDefaultFlowsheetID('');
  if cbxFlowsheets.ItemIndex > -1 then
    if cbxFlowsheets.Items.Objects[cbxFlowsheets.ItemIndex] <> nil then
      if cbxFlowsheets.Items.Objects[cbxFlowsheets.ItemIndex].GetInterface(ICPFlowsheet, aFlowsheet) then
        fOptions.setDefaultFlowsheetID(aFlowsheet.getID);

  fOptions.setConfirmOnClose(ckbxConfirmOnClose.Checked);
  fOptions.setAutoRefreshEnabled(ckbxRefreshEnabled.Checked);
  fOptions.setAutoRefreshInterval(udRefreshSeconds.Position);
  fOptions.setUsePatientLocation(ckbxUsePatientLocation.Checked);
  if cbxLocations.ItemIndex > 0 then
    begin
      cbxLocations.Items.Objects[cbxLocations.ItemIndex].GetInterface(ICPLocation, aLocation);
      fOptions.setDefaultLocationID(aLocation.getID);
    end
  else
    begin
      fOptions.setDefaultLocationID('');
    end;

  if TCPLookAndFeel(rgSkin.ItemIndex) <> fOptions.getLookAndFeel then
    fOptions.setLookAndFeel(TCPLookAndFeel(rgSkin.ItemIndex));

  for i := Low(OBSERVATION_RANGE_ORDER) to High(OBSERVATION_RANGE_ORDER) do
    begin
      fOptions.setFlowsheetItemFont(OBSERVATION_RANGE_ORDER[i], Samples[i].Font);
      fOptions.setFlowsheetItemColor(OBSERVATION_RANGE_ORDER[i], Samples[i].Color);
    end;
  fOptions.Save;
  acApply.Enabled := False;
  btnApply.Default := False;
  btnOK.Default := True;
end;

procedure TfrmUserOptions.acChangeMadeExecute(Sender: TObject);
var
  aSender: string;
begin
  if not fLoading then
    begin
      aSender := Sender.ClassName;
      acApply.Enabled := True;
      btnOK.Default := False;
      btnApply.Default := True;
    end;
end;

procedure TfrmUserOptions.acOKExecute(Sender: TObject);
begin
  acApply.Execute;
  ModalResult := mrOK;
end;

procedure TfrmUserOptions.cbxElementsChange(Sender: TObject);
begin
  if (cbxElements.ItemIndex > -1) and (not fLoading) then
    try
      fLoading := True;
      with Samples[cbxElements.ItemIndex] do
        begin
          ckbxBold.Checked := fsBold in Font.Style;
          ckbxItalics.Checked := fsItalic in Font.Style;
          ckbxUnderline.Checked := fsUnderline in Font.Style;
          clrbxForeground.Selected := Font.Color;
          clrbxBackground.Selected := Color;
        end;
      acChangeMade.Execute;
    finally
      fLoading := False;
    end
  else
    begin
    end;
end;

procedure TfrmUserOptions.ckbxRefreshEnabledClick(Sender: TObject);
begin
  edtRefreshSeconds.Enabled := ckbxRefreshEnabled.Checked;
  udRefreshSeconds.Enabled := ckbxRefreshEnabled.Checked;
  lblRefreshSeconds.Enabled := ckbxRefreshEnabled.Checked;
  acChangeMade.Execute;
end;

procedure TfrmUserOptions.udRefreshSecondsClick(Sender: TObject; Button: TUDBtnType);
begin
  edtRefreshSeconds.Text := IntToStr(udRefreshSeconds.Position);
  acChangeMade.Execute;
end;

procedure TfrmUserOptions.edtRefreshSecondsExit(Sender: TObject);
begin
  try
    if (StrToInt(edtRefreshSeconds.Text) > udRefreshSeconds.Max) or
      (StrToInt(edtRefreshSeconds.Text) < udRefreshSeconds.Min) then
      begin
        MessageDlg(Format(
          'Seconds between Auto-Refresh must be a value between %d and %d.',
          [udRefreshSeconds.Min, udRefreshSeconds.Max]), mtInformation, [mbOk], 0);
        edtRefreshSeconds.Text := IntToStr(fOptions.getAutoRefreshInterval);
        edtRefreshSeconds.SetFocus;
      end
    else
      begin
        udRefreshSeconds.Position := StrToInt(edtRefreshSeconds.Text);
        acChangeMade.Execute;
      end;
  except
    MessageDlg('Invalid value entered for ''Seconds between Auto-Refresh''.', mtError, [mbOk], 0);
    edtRefreshSeconds.Text := IntToStr(fOptions.getAutoRefreshInterval);
    edtRefreshSeconds.SetFocus;
  end;
end;

procedure TfrmUserOptions.edtRefreshSecondsKeyPress(Sender: TObject; var Key: Char);
begin
  acChangeMade.Execute;
end;

procedure TfrmUserOptions.FontChange(Sender: TObject);
begin
  if (cbxElements.ItemIndex > -1) and (not fLoading) then
    with Samples[cbxElements.ItemIndex].Font do
      begin
        if ckbxBold.Checked then
          Style := Style + [fsBold]
        else
          Style := Style - [fsBold];

        if ckbxItalics.Checked then
          Style := Style + [fsItalic]
        else
          Style := Style - [fsItalic];

        if ckbxUnderline.Checked then
          Style := Style + [fsUnderline]
        else
          Style := Style - [fsUnderline];

        Color := clrbxForeground.Selected;
        Samples[cbxElements.ItemIndex].Color := clrbxBackground.Selected;
        Samples[cbxElements.ItemIndex].Invalidate;
        Application.ProcessMessages;
      end
  else
    begin
    end;
  acChangeMade.Execute;
end;

procedure TfrmUserOptions.FormCreate(Sender: TObject);
var
  aRootNode: TTreeNode;
begin
  tbshtFlowsheetColors.TabVisible := False;
  tbshtGeneralOptions.TabVisible := False;
  tbshtIntro.TabVisible := False;
  lblIntro.Font.Style := [fsBold];
  lblHeader.Caption := 'Options';
  lblHeader.Font.Style := [fsBold];
  lblGeneralHeader.Caption := tbshtGeneralOptions.Caption;
  lblGeneralHeader.Font.Style := [fsBold];
  lblFlowsheetHeader.Caption := tbshtFlowsheetColors.Caption;
  lblFlowsheetHeader.Font.Style := [fsBold];
  aRootNode := tvOptions.Items.AddChild(nil, 'Options');
  tvOptions.Items.AddChildObject(aRootNode, tbshtGeneralOptions.Caption, tbshtGeneralOptions);
  tvOptions.Items.AddChildObject(aRootNode, tbshtFlowsheetColors.Caption, tbshtFlowsheetColors);
  aRootNode.Expand(True);
  pgctrl.ActivePage := tbshtIntro;
end;

procedure TfrmUserOptions.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    ModalResult := mrCancel;
end;

procedure TfrmUserOptions.tvOptionsChange(Sender: TObject; Node: TTreeNode);
begin
  if Assigned(Node.Data) then
    pgctrl.ActivePage := TTabSheet(Node.Data)
  else
    pgctrl.ActivePage := tbshtIntro;
end;

procedure TfrmUserOptions.tvOptionsGetImageIndex(Sender: TObject; Node: TTreeNode);
begin
  if Node = tvOptions.TopItem then
    if Node.Expanded then
      begin
        Node.ImageIndex := USER_OPTIONS_IMAGES[2];
        Node.SelectedIndex := USER_OPTIONS_IMAGES[2];
      end
    else
      begin
        Node.ImageIndex := USER_OPTIONS_IMAGES[3];
        Node.SelectedIndex := USER_OPTIONS_IMAGES[3]
      end
  else if Assigned(Node.Data) then
    if TObject(Node.Data).ClassNameIs('TTabSheet') then
      begin
        Node.ImageIndex := USER_OPTIONS_IMAGES[TTabsheet(Node.Data).PageIndex];
        Node.SelectedIndex := USER_OPTIONS_IMAGES[TTabsheet(Node.Data).PageIndex];
      end
    else
  else
    begin
    end;
end;

end.

