unit mAlarmView;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ComCtrls,
  ToolWin,
  ActnList,
  oCliO,
  uGlobalVars,
  ImgList,
  ExtCtrls,
  Menus,
  VA508AccessibilityManager,
  fImageList, VA508ImageListLabeler, System.Actions;

type
  TfraAlarmView = class(TForm, ICPView)
    ActionList1: TActionList;
    acRefresh: TAction;
    acDeactivate: TAction;
    tBarGridDateSelector: TToolBar;
    ToolButton4: TToolButton;
    ToolButton3: TToolButton;
    ToolButton5: TToolButton;
    tbtnPurge: TToolButton;
    tbtnVerify: TToolButton;
    tbtnRescind: TToolButton;
    acUpdate: TAction;
    acNewAlarm: TAction;
    acTestAlarms: TAction;
    acClearAlarm: TAction;
    Bevel2: TBevel;
    lvAlarms: TListView;
    Bevel1: TBevel;
    pmnView: TPopupMenu;
    Refresh2: TMenuItem;
    NewAlarm2: TMenuItem;
    Update2: TMenuItem;
    ClearAlarm2: TMenuItem;
    Deactivate2: TMenuItem;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    VA508ImageListLabeler1: TVA508ImageListLabeler;
    procedure acRefreshExecute(Sender: TObject);
    procedure acUpdateExecute(Sender: TObject);
    procedure acNewAlarmExecute(Sender: TObject);
    procedure acDeactivateExecute(Sender: TObject);
    procedure lvAlarmsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure acTestAlarmsExecute(Sender: TObject);
    procedure lvAlarmsGetImageIndex(Sender: TObject; Item: TListItem);
    procedure acClearAlarmExecute(Sender: TObject);
    procedure lvAlarmsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
  private
    fCliOAlarmList: ICliOAlarmList;

    FMenuItem: TMenuItem;

    procedure ViewInitialize; stdcall;
    procedure ViewActivate; stdcall;
    procedure ViewReload; stdcall;
    procedure ViewDeactivate; stdcall;
    function  ViewMenu: TMenuItem; stdcall;
    function  ViewHelpContext: integer; stdcall;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

function NewAlarmViewFrame(aOwner: TWinControl; aCliOAlarmList: ICliOAlarmList): TfraAlarmView;

implementation

uses
  fAlarmSettings,
  oInterfaces,
  u508Dialogs;

{$R *.dfm}

function NewAlarmViewFrame(aOwner: TWinControl; aCliOAlarmList: ICliOAlarmList): TfraAlarmView;
begin
  Result := TfraAlarmView.Create(aOwner);
  with Result do
    begin
      Parent := aOwner;
      Align := alClient;
      fCliOAlarmList := aCLiOAlarmList;
    end;
end;

procedure TfraAlarmView.ViewActivate;
begin
  Visible := True;
  {Update to the main loaders methods}
  fCliOAlarmList.OnCliOAlarmSet := MyAlarmList.OnCliOAlarmSet;
  fCliOAlarmList.OnCliOAlarmClear := MyAlarmList.OnCliOAlarmClear;
  ViewReload;
end;

procedure TfraAlarmView.ViewDeactivate;
begin
  Visible := False;
end;

function TfraAlarmView.ViewHelpContext: integer;
begin
  result := HelpContext;
end;

procedure TfraAlarmView.ViewInitialize;
begin
  acClearAlarm.Enabled := false;
end;

procedure TfraAlarmView.ViewReload;
var
  i: integer;
  aCliOAlarm: ICliOAlarm;
begin
  lvAlarms.Items.Clear;
  if Assigned(MyPatient) then
    begin
      fCliOAlarmList.Load(MyPatient.getDFN);
      for i := 0 to fCliOAlarmList.getAlarmCount - 1 do
        begin
          fCliOAlarmList.getAlarm(i).QueryInterface(ICliOAlarm, aCliOAlarm);
          with aCliOAlarm.AddToListView(lvAlarms) do
            begin
              Caption := aCliOAlarm.getTerm.getTerm;
              SubItems.Add(aCliOAlarm.GetAlarmSettings);
              SubItems.Add(aCliOAlarm.getActivationComment);
            end;
        end;
    end;
  lvAlarms.Invalidate;
  acNewAlarm.Enabled := not MyUser.HasRole(ROLE_CLIO_READ_ONLY);
  acUpdate.Enabled :=  (not MyUser.HasRole(ROLE_CLIO_READ_ONLY)) and (lvAlarms.Selected <> nil);
  acClearAlarm.Enabled :=  (not MyUser.HasRole(ROLE_CLIO_READ_ONLY)) and (lvAlarms.Selected <> nil);
  acDeactivate.Enabled :=  (not MyUser.HasRole(ROLE_CLIO_READ_ONLY)) and (lvAlarms.Selected <> nil);
  acTestAlarms.Enabled :=  not MyUser.HasRole(ROLE_CLIO_READ_ONLY);
end;

procedure TfraAlarmView.acClearAlarmExecute(Sender: TObject);
var
  aCliOAlarm: ICliOAlarm;
  i: integer;
begin
  if lvAlarms.SelCount > 0 then
    for i := lvAlarms.Items.Count - 1 downto 0 do
      try
        if lvAlarms.Items[i].Selected then
          if TObject(lvAlarms.Items[i].Data).GetInterface(ICliOAlarm, aCliOAlarm) then
            aCliOAlarm.Clear;
      except
        MessageDlg508('Error Clearing Alarm' + lvAlarms.Items[i].Caption, mtError, [mbOk], 0);
      end
  else
    MessageDlg508('Please select alarms to clear first.', mtInformation, [mbOk], 0);
  acRefresh.Execute;
end;

procedure TfraAlarmView.acDeactivateExecute(Sender: TObject);
var
  aCliOAlarm: ICliOAlarm;
  aComment: string;
  i: integer;
begin
  if lvAlarms.SelCount > 0 then
    if InputQuery('Deactivate Alarm', 'Comment:', aComment) then
      if aComment <> '' then
        begin
          for i := lvAlarms.Items.Count - 1 downto 0 do
            if lvAlarms.Items[i].Selected then
              try
                if TObject(lvAlarms.Items[i].Data).GetInterface(ICliOAlarm, aCliOAlarm) then
                  aCliOAlarm.Deactivate(MyUser, aComment);
              except
                MessageDlg508('Error Deactivating ' + lvAlarms.Items[i].Caption, mtError, [mbOk], 0);
              end;
        end
      else
        MessageDlg508('Comments are required to deactivate and alarm.', mtInformation, [mbOk], 0)
    else
      MessageDlg508('Alarm deactivation cancelled.', mtInformation, [mbOk], 0)
  else
    MessageDlg508('Please select at least one alarm first.', mtInformation, [mbOk], 0);

  acRefresh.Execute;
end;

procedure TfraAlarmView.acNewAlarmExecute(Sender: TObject);
begin
  EditPatientAlarm(nil);
  acRefresh.Execute;
end;

procedure TfraAlarmView.acRefreshExecute(Sender: TObject);
begin
  ViewReload;
end;

procedure TfraAlarmView.acTestAlarmsExecute(Sender: TObject);
var
  aCliOAlarm: ICliOAlarm;
  aValue: string;
begin
  if Assigned(lvAlarms.Selected) then
    begin
      aCliOAlarm := fCliOAlarmList.getAlarm(lvAlarms.Selected.SubItems[2]);
      if Assigned(aCliOAlarm) then
        begin
          if InputQuery('Alarm Test', 'Value to Test', aValue) then
            MyAlarmList.Execute(aValue, aCliOAlarm.getTerm, aCLiOAlarm.getUnit);
        end
      else
        MessageDlg508('Unable to access alarm settings', mtError, [mbOk], 0);
    end;
end;

procedure TfraAlarmView.acUpdateExecute(Sender: TObject);
var
  aCliOAlarm: ICliOAlarm;
begin
  if Assigned(lvAlarms.Selected) and not MyUser.HasRole(ROLE_CLIO_READ_ONLY) then
    if Assigned(lvAlarms.Selected.Data) then
      if TObject(lvAlarms.Selected.Data).GetInterface(ICliOAlarm, aCliOAlarm) then
        begin
          EditPatientAlarm(aCliOAlarm);
          with lvAlarms.Selected do
            begin
              Caption := aCliOAlarm.getTerm.getTerm;
              SubItems.Clear;
              SubItems.Add(aCliOAlarm.GetAlarmSettings);
              SubItems.Add(aCliOAlarm.getActivationComment);
            end;
        end;
end;

constructor TfraAlarmView.Create(AOwner: TComponent);
var
  Item: TMenuItem;
begin
  inherited;

  FMenuItem := TMenuItem.Create(nil);
  FMenuItem.Caption := 'Alarms';

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acRefresh;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acNewAlarm;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acUpdate;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acClearAlarm;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acDeactivate;
end;

destructor TfraAlarmView.Destroy;
begin
  FreeAndNil(FMenuItem);
  inherited;
end;

procedure TfraAlarmView.FormShow(Sender: TObject);
begin
  lvAlarms.Selected := nil;
end;

procedure TfraAlarmView.lvAlarmsGetImageIndex(Sender: TObject; Item: TListItem);
var
  aCliOAlarm: ICliOAlarm;
begin
  if Assigned(Item.Data) then
    if TObject(Item.Data).GetInterface(ICliOAlarm, aCliOAlarm) then
      if aCliOAlarm.getActivated then
        Item.ImageIndex := 121
      else
        Item.ImageIndex := -1
    else
      Item.ImageIndex := -1
  else
    Item.ImageIndex := -1;
end;

procedure TfraAlarmView.lvAlarmsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_RETURN) then
  begin
    acUpdate.Execute;
  end;
end;

procedure TfraAlarmView.lvAlarmsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  Alarm: ICliOAlarm;
begin
  if not MyUser.HasRole(ROLE_CLIO_READ_ONLY) then
  begin
    acUpdate.Enabled := Selected;
    acDeactivate.Enabled := Selected;
    if assigned(Item) then
      if Assigned(Item.Data) then
        if TObject(Item.Data).GetInterface(ICliOAlarm, Alarm) then
         acClearAlarm.Enabled := Alarm.getActivated;
  end;
end;

function  TfraAlarmView.ViewMenu: TMenuItem; stdcall;
begin
  Result := FMenuItem;
end;

end.

