unit mEditObservationBattery;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ActnList,
  ExtCtrls,
  StdCtrls,
  ComCtrls,
  XMLDoc,
  XMLIntf,
  oInterfaces,
  oCliO,
  oCliOTypes,
  VA508AccessibilityManager;

type
  TOnSaveBattery = function(aXMLDocument: IXMLDocument): boolean of object;

  TfraEditObservationBattery = class(TForm)
    pnlDemographics: TPanel;
    lblComments: TLabel;
    lblDate: TLabel;
    lblTime: TLabel;
    lblLocation: TLabel;
    dtpDateObserved: TDateTimePicker;
    dtpTimeObserved: TDateTimePicker;
    edtComment: TEdit;
    cbxLocation: TComboBox;
    pnlCommands: TPanel;
    btnCancel: TButton;
    btnSave: TButton;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Shape1: TShape;
    bvlTools: TBevel;
    lblHeader: TVA508StaticText;
    v508AccessMgr: TVA508AccessibilityManager;
    pnlObservations: TPanel;
    pnlLabels: TPanel;
    lblValue: TLabel;
    lblObservation: TLabel;
    Shape2: TShape;
    scbxForFrames: TScrollBox;
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
  protected
    fCancelEdit: TNotifyEvent;
    fSaveBattery: TOnSaveBattery;
    fView: ICPFlowsheetView;
    fCliOFiler: ICliOFiler;
    procedure fOnCancelEdit(Sender: TObject);
    function IsValid: boolean;
  public
    procedure setView(aView: ICPFlowsheetView);
    procedure setOnCancelEdit(aOnCancelEdit: TNotifyEvent);
    procedure setOnSaveBattery(aOnSaveBattery: TOnSaveBattery);
  end;

function NewEditObservationBatteryFrame(aOwner: TWinControl; aView: ICPFlowsheetView; aCliOFiler: ICliOFiler): TfraEditObservationBattery;
procedure AddObservations(aView: ICPFlowsheetView; aCliOFiler: ICliOFiler; onCancel: TNotifyEvent);

implementation

uses
  uGlobalVars,
  mEditObservationValue,
  oVersionInformation,
  DateUtils,
  oDBController_Patient,
  oCPTypes,
  u508Dialogs;

{$R *.dfm}

function NewEditObservationBatteryFrame(aOwner: TWinControl; aView: ICPFlowsheetView; aCliOFiler: ICliOFiler): TfraEditObservationBattery;
var
  tt: TTermType;
  //M, D, Y, hh, mm, ss, ms: word;
//  aCPLocation: ICPLocation;
  aMinute: integer;
//  aRound: Double;
//  AItem: TVA508AccessibilityItem;
  lbl: TLabel;
begin
  Result := TfraEditObservationBattery.Create(aOwner);
  with Result do
    begin
      Parent := aOwner;
      Align := alClient;
      MyPatientSource.LoadLocations(cbxLocation.Items);
      {Set to patient location???}
      if MyUserOptions.getUsePatientLocation then
        cbxLocation.ItemIndex := MyPatientSource.GetLocationIndexByName(cbxLocation.Items, MyPatient.getWardLocation);

      {If still no location and MyUser has a default, try that}
      if (MyUserOptions.getDefaultLocationID <> '') and (cbxLocation.ItemIndex < 0) then
        cbxLocation.ItemIndex := MyPatientSource.GetLocationIndexByID(cbxLocation.Items, MyUserOptions.getDefaultLocationID);

      lblHeader.Font.Style := [fsBold];
      if aView.getSupplementalID = '' then
        lblHeader.Caption := Format('%s - %s', [aView.getDisplayName, aView.getComment])
      else
        lblHeader.Caption := aView.getSupplementalTitle;
      pnlLabels.Font.Style := [fsBold];
      aMinute := MinuteOfTheDay(Now);
      { This is the one we tried to implement but is caused huge diffs in the bigger intervals
      aRound := aView.getDisplayInterval / 2;
      aMinute := trunc((aMinute + aRound) / aView.getDisplayInterval) *
        aView.getDisplayInterval;
     }

      { This method will round to the closest 10 minute mark
      aMinute := trunc((aMinute + 5) / 10) * 10;
      }

      dtpDateObserved.DateTime := IncMinute(trunc(Now), aMinute);
      dtpTimeObserved.DateTime := IncMinute(trunc(Now), aMinute);
      //DecodeDateTime(Now, Y, M, D, hh, mm, ss, ms);
      //dtpDateObserved.DateTime := EncodeDate(Y, M, D);
      //dtpTimeObserved.DateTime := EncodeDateTime(Y, M, D, hh, mm, 0, 0);
      for tt := High(TTermType) downto Low(TTermType) do
        if tt in TERMQUALIFIERS then
          begin
            lbl := TLabel.Create(pnlLabels);
            with lbl do
              begin
                Name := Format('%s_%d', [ClassName, Ord(tt)]);
                Parent := pnlLabels;
                Left := 85 * Ord(tt); {Modified after removal of Null Flavor CQ705}
                AutoSize := False;
                //Layout := tlCenter;
                Width := 85; {Modified after removal of Null Flavor CQ705}
                Align := alRight;
                Caption := QUALNAME[tt];

              end;
          end;

      setView(aView);
      fCancelEdit := fOnCancelEdit;
      fCliOFiler := aCliOFiler;
      v508AccessMgr.RefreshComponents;
    end;
end;

procedure AddObservations(aView: ICPFlowsheetView; aCliOFiler: ICliOFiler; onCancel: TNotifyEvent);
var
  aForm: TForm;
  aFrame: TfraEditObservationBattery;
begin
  Application.CreateForm(TForm, aForm);
  try
    aFrame := NewEditObservationBatteryFrame(aForm, aView, aClioFiler);
    aFrame.Parent := aForm;
    aFrame.Align := alClient;
    aFrame.Show;
    aFrame.setOnCancelEdit(onCancel);
    aFrame.bvlTools.Visible := True;
    aFrame.lblHeader.Caption := '   ' + aFrame.lblHeader.Caption;
    aFrame.lblHeader.Visible := False;
    aFrame.pnlDemographics.ParentBackground := False;
    aFrame.pnlDemographics.Color := clSilver;
    aForm.Position := poScreenCenter;
    aForm.Width := 800;
    aForm.Height := 600;
    aForm.Caption := 'Input: ' + aFrame.lblHeader.Caption;
    aForm.Color := clWindow;
    aForm.ShowModal;
  finally
    aForm.Free;
  end;
end;

{ TfraEditObservationBattery }

procedure TfraEditObservationBattery.setOnCancelEdit(aOnCancelEdit: TNotifyEvent);
begin
  fCancelEdit := aOnCancelEdit;
end;

procedure TfraEditObservationBattery.setOnSaveBattery(aOnSaveBattery: TOnSaveBattery);
begin
  fSaveBattery := aOnSaveBattery;
  btnSave.Enabled := Assigned(fSaveBattery);
end;

procedure TfraEditObservationBattery.fOnCancelEdit(Sender: TObject);
begin
  MessageDlg508('Error, no way to cancel this edit', mtError, [mbOk], 0);
end;

procedure TfraEditObservationBattery.SetView(aView: ICPFlowsheetView);
var
  i: integer;
  aViewTerm: ICPFlowsheetViewTerm;
begin
  fView := aView;
  for i := fView.getViewTermCount - 1 downto 0 do
    begin
      aViewTerm := aView.getViewTermByIndex(i);
      if aViewTerm.getDisplayOnly = False then
        with NewEditObservationValueFrame(scbxForFrames, aViewTerm) do
          begin
            Align := alTop;
            if frac(scbxForFrames.ComponentCount / 2) > 0 then
              begin
                ParentBackground := False;
                Color := clInfoBk;
              end;

            {Hide boxes with a view filter value - we'll save them automatically}
            if Assigned(aView.getDefaultMethod) then
              fQualifier[ttMethod].Visible := False;

            if Assigned(aView.getDefaultPosition) then
              fQualifier[ttPosition].Visible := False;

            if Assigned(aView.getDefaultLocation) then
              fQualifier[ttLocation].Visible := False;

            if Assigned(aView.getDefaultProduct) then
              fQualifier[ttProduct].Visible := False;
            visible := true;
          end;
    end;
end;

procedure TfraEditObservationBattery.btnCancelClick(Sender: TObject);
begin
  fCancelEdit(Sender);
end;

procedure TfraEditObservationBattery.btnSaveClick(Sender: TObject);
var
  aDoc: ICliOFilerDoc;
  aObs: ICliOFilerObs;
  aSet: ICliOFilerSet;
  i: integer;
  aCPLocation: ICPLocation;
  aLog: TStringList;
  aCount: integer;
begin
  {Make sure the date time fits the page if it is supplemental}
  if fView.getSupplementalID <> '' then
    begin
      if trunc(dtpDateObserved.DateTime) + frac(dtpTimeObserved.DateTime) < fView.getActivatedDateTime then
        begin
          MessageDlg508('Date time of observation is before page activation date/time.', mtError, [mbOk], 0);
          Exit;
        end;

      if fView.getDeactivatedByID <> '' then
        if trunc(dtpDateObserved.DateTime) + frac(dtpTimeObserved.DateTime) > fView.getDeactivatedDateTime then
          begin
            MessageDlg508('Date time of observation is after page deactivation date/time.', mtError, [mbOk], 0);
            Exit;
          end;
    end;

  {Check if any fields are entered and required fields}
  aCount := 0;
  aLog := TStringList.Create;

  for i := 0 to scbxForFrames.ComponentCount - 1 do
    if scbxForFrames.Components[i].ClassNameIs('TfraEditObservationValue') then
      with TfraEditObservationValue(scbxForFrames.Components[i]) do
        begin
          if GetHasValue then
            inc(aCount);

          if GetIsRequired and (not GetHasValue) then
            aLog.Add(Format('%2d. Field ''%s'' is required.', [aLog.Count + 1, GetTermDisplayName]));
        end;

  if aCount = 0 then
    begin
      MessageDlg508('Input form is blank, nothing to save.' + #13 + #13 + 'Click on the Cancel button if trying to exit editor.', mtInformation, [mbOk], 0);
      FreeAndNil(aLog);
      Exit;
    end
  else
    if aLog.Count > 0 then
      begin
        MessageDlg508('Input form is missing required data.' + #13 + #13 + aLog.Text, mtInformation, [mbOk], 0);
        FreeAndNil(aLog);
        Exit;
      end;

  FreeAndNil(aLog);

  {Make sure we have a location}
  if cbxLocation.ItemIndex < 0 then
    begin
      MessageDlg508('Location must be selected to file observations.', mtInformation, [mbOk], 0);
      Exit;
    end;

  {Ok, try to file all this stuff}

  aDoc := fCLiOFiler.getNewDocument(MyUser);

  //ShowMessage('Debug Peek #1' + #13 + FormatXMLData(aDoc.getXML));

  aSet := aDoc.CreateNewSet;
  with aSet do
    begin
      setEnteredDateTime(Now);
      setEnteredByID(MyUser.getDUZ);
      setComment(edtComment.Text);
      setIsPublic(True);
    end;

  //ShowMessage('Debug Peek #2' + #13 + FormatXMLData(aDoc.getXML));

  {Load the observation values}
  for i := 0 to scbxForFrames.ComponentCount - 1 do
    if scbxForFrames.Components[i].ClassNameIs('TfraEditObservationValue') then
      TfraEditObservationValue(scbxForFrames.Components[i]).LoadXML(aDoc);

  //ShowMessage('Debug Peek #3' + #13 + FormatXMLData(aDoc.getXML));

  {Now drop through all observations and update with the proper date/time}
  dtpDateObserved.DateTime := trunc(dtpDateObserved.Date) + Frac(dtpTimeObserved.Time);

  if dtpDateObserved.DateTime > now then
    begin
      MessageDlg508('Date/time of observation cannot be in the future.', mtError, [mbOk], 0);
      Exit;
    end;

  if cbxLocation.ItemIndex > -1 then
    cbxLocation.Items.Objects[cbxLocation.ItemIndex].GetInterface(ICPLocation, aCPLocation)
  else
    aCPLocation := nil;

  aObs := aDoc.getFirstObs;
  while Assigned(aObs) do
    begin
      with aObs do
        begin
          if Assigned(aCPLocation) then
            setFacilityLocationID(aCPLocation.getID)
          else
            setFacilityLocationID('');
          setPatientID(MyPatient.getDFN);
          setObservedDateTime(dtpDateObserved.DateTime);
          setObservedByID(MyUser.getDUZ);
          setEnteredDateTime(Now);
          setEnteredByID(MyUser.getDUZ);
          AddToSet(aSet.getID);
          AddToSet(fView.getDefaultSetID);
        end;
      aObs := aDoc.getNextObs(aObs);
    end;

  try
    fCliOFiler.Execute(aDoc);
    fCancelEdit(Sender);
  except
    MessageDlg508(fCliOFiler.getErrorLog, mtError, [mbOk], 0);
  end;
end;

function TfraEditObservationBattery.IsValid: boolean;
var
  aLog: TStringList;
  i: integer;
begin
  aLog := TStringList.Create;
  try
    for i := 0 to scbxForFrames.ComponentCount - 1 do
      TfraEditObservationValue(scbxForFrames.Components[i]).Validate(aLog);

    if aLog.Count > 0 then
      MessageDlg508('The following errors were reported' + #13 + aLog.Text, mtError, [mbOk], 0)
    else
      MessageDlg508('Woo Hoo, all clear', mtInformation, [mbOk], 0);

    Result := (aLog.Count = 0);
  finally
    FreeAndNil(aLog);
  end;
end;

end.

