unit mEditObservationValue;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 31 $  $Modtime: 12/29/10 12:32p $
*       Developer:    dan.petit@va.gov
*       Site:         Hines OIFO
*
*       Description:  Edits a single observation value.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Modernization/CliO/source/CP Flowsheets/mEditObservationValue.pas $
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Buttons,
  oCliO,
  oCliOTypes,
  oInterfaces,
  XMLIntf,
  ExtCtrls,
  CheckLst,
  VA508AccessibilityManager;

type
  TfraEditObservationValue = class(TForm)
    pnlValue: TPanel;
    lblTermDisplayNameLeft: TLabel;
    pnlLabel: TPanel;
    pnlQualifiers: TPanel;
    pnlMain: TPanel;
    pnlComment: TPanel;
    lblComment: TLabel;
    cbxComment: TComboBox;
    bvlBottom: TBevel;
    cklbxValue: TCheckListBox;
    rgValue: TRadioGroup;
    edtValue: TEdit;
    cbxValue: TComboBox;
    v508AccessMgr: TVA508AccessibilityManager;
    sbtnComments: TBitBtn;
    procedure cbxEscapeKey(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure pnlValueResize(Sender: TObject);
    procedure ClearComboBox(Sender: TObject; var Key: Char);
    procedure sbtnCommentsClick(Sender: TObject);
    procedure cbxValueEnter(Sender: TObject);
  private
    fViewTerm: ICPFlowsheetViewTerm;
    fTerm: ITerm;
    procedure QualifierEnter(Sender: TObject);
    procedure QualifierComboResize(Sender: TObject);
  public
    fQualifier: array[TTermType] of TComboBox;
    GUID: string; {Used to create the observation}
    ParentGUID: string; {Used for complex observations}
    function GetID(aTermType: TTermType): string;
    function GetHasValue: boolean;
    function GetIsRequired: boolean;
    function GetTermDisplayName: string;
    procedure SetDefault(aTermType: TTermType; aTerm: ITerm);
    procedure SetMandatory(aTermType: TTermType; aTerm: ITerm);
    procedure SetNotAllowed(aTermType: TTermType);
    procedure SetAsComplex;
    procedure Validate(aErrorLog: TStringList);
    procedure LoadXML(aFilerDocument: ICliOFilerDoc);
  end;

function NewEditObservationValueFrame(aOwner: TWinControl; aViewTerm: ICPFlowsheetViewTerm): TfraEditObservationValue;

implementation

uses
  oTerm,
  uGlobalVars,
  DateUtils,
  StrUtils,
  VA508AccessibilityRouter,
  uCP_GUI;

{$R *.dfm}

function NewEditObservationValueFrame(aOwner: TWinControl; aViewTerm: ICPFlowsheetViewTerm): TfraEditObservationValue;
var
  tt: TTermType;
  i: integer;
  aTerm: ITerm;
  aLabel: string;
//  fPickOne: Boolean;
//  Item: TVA508AccessibilityItem;
const
  USE_COMBO_BOX = True;

begin
//  fPickOne := False;
  Result := TfraEditObservationValue.Create(aOwner);
  with Result do
    begin
      fViewTerm := aViewTerm;
      pnlValue.BorderStyle := bsNone;
      pnlLabel.BorderStyle := bsNone;
      pnlQualifiers.BorderStyle := bsNone;
      pnlMain.BorderStyle := bsNone;
      pnlComment.BorderStyle := bsNone;
      fTerm := MyTerminology.GetTerm(aViewTerm.getAttributeID(ttObservation));
      //fTerm := aViewTerm.getAttributeTerm(ttObservation);
      GUID := NewGUID(True);
      Name := Format('f%s', [GUID]);
      Parent := aOwner;
      Top := (aOwner.ComponentCount * Height) + 1;
      TabOrder := 0;
      
      if frac(aOwner.ComponentCount / 2) > 0 then
        Color := cl3DLight
      else
        Color := clWindow;

      {Setup without comments showing!}
      lblComment.Font.Style := [fsBold];
      pnlComment.Visible := False;
      Height := pnlMain.Height + bvlBottom.Height;

      aLabel := aViewTerm.getDisplayName;
      if aViewTerm.getRequiredTerm then
        aLabel := aLabel + ' (required)';
      lblTermDisplayNameLeft.Font.Style := [fsBold];
      lblTermDisplayNameLeft.Caption := aLabel;

      edtValue.Hint := fTerm.getHelpText;
      rgValue.Hint := fTerm.getHelpText;
      cklbxValue.Hint := fTerm.getHelpText;

      i := 0;
      for tt := High(TTermType) downto Low(TTermType) do
        if tt in TERMQUALIFIERS then
          begin
            fQualifier[tt] := TComboBox.Create(Result);
            with fQualifier[tt] do
              begin
                Inc(i);
                Name := Format('%s_%d', [ClassName, i]);
                Parent := pnlQualifiers;
                Width := 80;  // was 60, removed Null Flavor drp 12-29-2010
                Left := pnlQualifiers.Width - (i * 85);  // was 65, removed Null Flavor drp 12-29-2010
                Top := 2;
                Anchors := [akTop, akRight];
                AutoCompleteDelay := 1000;
                pnlQualifiers.Width := (i * 85);  // was 65, removed Null Flavor drp 12-29-2010
                OnDropDown := QualifierComboResize;
                if not (tt = ttUnit) then
                  OnKeyPress := ClearComboBox;
                {
                508
                blj 19 March 2009 - We'll probably want to revisit this at some point.
                The 508 component set flatly refuses to read any text added to a
                TVA508AccessibilityItem that is added here.  So, we kind of had
                to do this the hard way.  If we can ever get the problem here
                figured out, we'll want to do:
                
                Item := v508AccessMgr.AccessData.Add;
                Item.Component := fQualifier[tt];
                Item.AccessText := QUALNAME[tt];
                }

                Hint := QUALNAME[tt];
                OnEnter := QualifierEnter;
              end;
          end;
      v508AccessMgr.RefreshComponents;

      Align := alTop; {Do this here so the comboboxes will follow their anchors}

      {
      Now go back and populate the comboboxes so they stay aligned
      I am not sure why they won't align correctly above but they don't :(
      }

      for tt := Low(TTermType) to High(TTermType) do
        if fQualifier[tt] <> nil then
          with fQualifier[tt] do
            begin
              Style := csDropDownList;
              fTerm.LoadListOfPairs(tt, Items, tdDisplayName);
              TabOrder := Ord(tt);
              Visible := False;
              //Visible := Items.Count > 0;
              (*
              if Items.Count = 1 then
                begin
                  ItemIndex := 0;
                  Style := csSimple;
                  Enabled := False;
                  Font.Style := [fsBold];
                end
              else
              *)
                //if Assigned(aViewTerm.getAttributeTerm(tt)) then
                //if aViewTerm.getAttributeID(tt) <> '' then
              if aViewTerm.getAttributeAllowed(tt) then
                begin
                  {See if the attribute is in the list and default to it}
                  fQualifier[tt].Visible := True;
                  if aViewTerm.getAttributeID(tt) <> '' then
                    for i := 0 to fQualifier[tt].Items.Count - 1 do
                      if fQualifier[tt].Items.Objects[i].GetInterface(ITerm, aTerm) then
                        if aTerm.getID = aViewTerm.getAttributeID(tt) then
                          fQualifier[tt].ItemIndex := i;
                  {If the attribute exists and was found and is mandatory, lock it down}
                  if fQualifier[tt].ItemIndex > -1 then
                    if aViewTerm.getAttributeMandatory(tt) then
                      begin
                        Style := csSimple;
                        Enabled := False;
                        Font.Style := [fsBold];
                      end
                end;
              //else {Leave it visible if it has items in it}
              //  fQualifier[tt].Visible := False;
            end;

      case fTerm.getDataType of
        dtPicklist:
          if (not fTerm.getMultiSelectPickList) then
            if fViewTerm.getDisplayInDropdown or ScreenReaderSystemActive then
              begin
                edtValue.Visible := False;
                cklbxValue.Visible := False;
                rgValue.Visible := False;
                cbxValue.Visible := True;
                cbxValue.Top := 3;
                cbxValue.Left := 3;
                cbxValue.Width := pnlValue.Width - 40;
                fTerm.LoadListOfPairs(ttValue, cbxValue.Items, tdDisplayName);
                pnlMain.Height := 40;
                cbxValue.Hint := Format('%s (You may only select 1 item.)', [rgValue.Hint]);
              end
            else
              begin
                edtValue.Visible := False;
                cklbxValue.Visible := False;
                cbxValue.Visible := False;
                rgValue.Visible := True;
                rgValue.Top := 3;
                rgValue.Left := 3;
                rgValue.Height := pnlValue.Height - 10;
                rgValue.Width := pnlValue.Width - 40;
                rgValue.Columns := aViewTerm.getDisplayColumns;
                fTerm.LoadListOfPairs(ttValue, rgValue.Items, tdDisplayName);
                pnlMain.Height := 95;
                rgValue.Hint := Format('%s (You may only select 1 item.)', [rgValue.Hint]);
              end
          else
            begin
              edtValue.Visible := False;
              cklbxValue.Visible := True;
              rgValue.Visible := False;
              cbxValue.Visible := False;
              cklbxValue.Top := 3;
              cklbxValue.Left := 3;
              cklbxValue.Height := pnlValue.Height - 10;
              cklbxValue.Width := pnlValue.Width - 40;
              cklbxValue.Columns := aViewTerm.getDisplayColumns;
              fTerm.LoadListOfPairs(ttValue, cklbxValue.Items, tdDisplayName);
              //cklbxValue.BorderStyle := bsSingle;
              cklbxValue.Hint := Format('%s (You may select 0-%d items.)', [cklbxValue.Hint, cklbxValue.Items.Count]);
              cklbxValue.Color := Result.Color;
              pnlMain.Height := 110;
            end;
        dtBoolean:
          begin
            if ScreenReaderSystemActive then
            begin
              edtValue.Visible := False;
              cklbxValue.Visible := False;
              rgValue.Visible := False;
              cbxValue.Visible := True;
              cbxValue.Top := 3;
              cbxValue.Left := 3;
              cbxValue.Width := pnlValue.Width - 40;
              cbxValue.Items.Clear;
              cbxValue.Items.Add(FTerm.getBooleanValueFalse);
              cbxValue.Items.Add(FTerm.getBooleanValueTrue);
              pnlMain.Height := 40;
            end
            else
            begin
              edtValue.Visible := False;
              cklbxValue.Visible := False;
              cbxValue.Visible := False;
              rgValue.Visible := True;
              rgValue.Top := 3;
              rgValue.Left := 3;
              rgValue.Height := pnlValue.Height - 10;
              rgValue.Width := pnlValue.Width - 40;
              rgValue.Visible := True;
              rgValue.Columns := 2;
              rgValue.Items.Clear;
              rgValue.Items.Add(fTerm.getBooleanValueFalse);
              rgValue.Items.Add(fTerm.getBooleanValueTrue);
              pnlMain.Height := 65;
            end;
          end;
        dtNumeric:
          begin
            edtValue.Visible := True;
            cklbxValue.Visible := False;
            rgValue.Visible := False;
            cbxValue.Visible := False;
            Height := 30;
            fQualifier[ttUnit].Visible := True;
            edtValue.Top := 3;
            edtValue.Left := 3;
            //edtValue.Height := pnlValue.Height - 10;
            edtValue.Width := pnlValue.Width - 40;
            pnlMain.Height := 40;
          end;
        dtFunction:
          begin
            edtValue.Visible := True;
            edtValue.ParentColor := True;
            edtValue.ReadOnly := True;
            cklbxValue.Visible := False;
            cbxValue.Visible := False;
            rgValue.Visible := False;
            Height := 30;
            fQualifier[ttUnit].Visible := False;
            edtValue.Top := 3;
            edtValue.Left := 3;
            edtValue.Width := pnlValue.Width - 40;
            pnlMain.Height := 40;
          end;
      else
        begin
          edtValue.Visible := True;
          cklbxValue.Visible := False;
          rgValue.Visible := False;
          cbxValue.Visible := False;
          Height := 30;
          fQualifier[ttUnit].Visible := False;
          edtValue.Top := 3;
          edtValue.Left := 3;
          edtValue.Height := pnlValue.Height - 10;
          edtValue.Width := pnlValue.Width - 40;
          pnlMain.Height := 40;
        end;
      end;
      pnlValueResize(nil);
      Result.Height := pnlMain.Top + pnlMain.Height + bvlBottom.Height;
      v508AccessMgr.RefreshComponents;
      Visible := true;
    end;
end;

{ TfraEditObservationValue }

procedure TfraEditObservationValue.cbxEscapeKey(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    TComboBox(Sender).ItemIndex := -1;
end;

procedure TfraEditObservationValue.cbxValueEnter(Sender: TObject);
begin
  // 508 workaround: for some reason, the 508 components aren't reading the label.
  GetScreenReader.Speak(lblTermDisplayNameLeft.Caption);
end;

procedure TfraEditObservationValue.ClearComboBox(Sender: TObject; var Key: Char);
begin
  if Key = #27 then
    (Sender as TComboBox).ItemIndex := -1;
end;

function TfraEditObservationValue.GetHasValue: boolean;
var
  i: Integer;
  cklbxSelected: boolean;
begin
  {
  The cklbxValue.SelCount wasn't working the way we wanted it to.  So, we have
  to go through the old fashioned way and see if anything is checked
  }
  cklbxSelected := false;
  for I := 0 to cklbxValue.Count - 1 do
  begin
    cklbxSelected := cklbxValue.Checked[I];
    if cklbxSelected then
      break;
  end;

  Result := (
    (cklbxSelected) or
    (edtValue.Text <> '') or
    (rgValue.ItemIndex > -1) or
    (cbxValue.ItemIndex > -1)
  );
end;

function TfraEditObservationValue.getID(aTermType: TTermType): string;
var
  aTerm: ITerm;
begin
  Result := '';
  if aTermType = ttObservation then
    Result := fTerm.getID
  else if fQualifier[aTermType] <> nil then
    if fQualifier[aTermType].ItemIndex > -1 then
      if fQualifier[aTermType].Items.Objects[fQualifier[aTermType].ItemIndex].GetInterface(ITerm, aTerm) then
        Result := aTerm.getID;
end;

function TfraEditObservationValue.GetIsRequired: boolean;
begin
  Result := fViewTerm.getRequiredTerm;
end;

function TfraEditObservationValue.GetTermDisplayName: string;
begin
  Result := fViewTerm.getDisplayName;
end;

procedure TfraEditObservationValue.LoadXML(aFilerDocument: ICliOFilerDoc);
var
  aObs: ICliOFilerObs;
  aTerm: ITerm;
  i: integer;

  procedure LoadQualifiers(aCliOFilerObs: ICliOFilerObs);
  var
    tt: TTermType;
  begin
    for tt := Low(TTermType) to High(TTermType) do
      if tt in TERMQUALIFIERS then
        if GetID(tt) <> '' then
          aCliOFilerObs.setQualifier(tt, getID(tt));
    aCliOFilerObs.setComment(cbxComment.Text);
  end;

begin
  case fTerm.getDataType of
    dtPickList:
      begin
        if fTerm.getMultiSelectPickList then
          begin
            for i := 0 to cklbxValue.Items.Count - 1 do
              if cklbxValue.Checked[i] then
                begin
                  cklbxValue.Items.Objects[i].GetInterface(ITerm, aTerm);
                  aObs := aFilerDocument.CreateNewObs('');
                  aObs.setTermID(fTerm.getID);
                  aObs.setValue(aTerm.getID);
                  LoadQualifiers(aObs);
                end
          end
        else if rgValue.ItemIndex > -1 then
          begin
            rgValue.Items.Objects[rgValue.ItemIndex].GetInterface(ITerm, aTerm);
            aObs := aFilerDocument.CreateNewObs('');
            aObs.setTermID(fTerm.getID);
            aObs.setValue(aTerm.getID);
            LoadQualifiers(aObs);
          end
          else if cbxValue.ItemIndex > -1 then
          begin
            cbxValue.Items.Objects[cbxValue.ItemIndex].GetInterface(ITerm, aTerm);
            aObs := aFilerDocument.CreateNewObs('');
            aObs.setTermID(fTerm.getID);
            aObs.setValue(aTerm.getID);
            LoadQualifiers(aObs);
          end;
      end;
    dtBoolean:
      begin
        if rgValue.ItemIndex > -1 then
          begin
            aObs := aFilerDocument.CreateNewObs('');
            aObs.setTermID(fTerm.getID);
            aObs.setValue(IntToStr(rgValue.ItemIndex));
            LoadQualifiers(aObs);
          end;
      end;
  else
    if edtValue.Text <> '' then
      begin
        aObs := aFilerDocument.CreateNewObs('');
        aObs.setTermID(fTerm.getID);
        aObs.setValue(edtValue.Text);
        LoadQualifiers(aObs);
      end;
  end;
end;

procedure TfraEditObservationValue.pnlValueResize(Sender: TObject);
begin
  (*
  cklbxValue.Columns := trunc(cklbxValue.Items.Count / 5);
  if frac(cklbxValue.Items.Count / 5) > 0 then
    cklbxValue.Columns := cklbxValue.Columns + 1;

  rgValue.Columns := trunc(rgValue.Items.Count / 3);
  if frac(rgValue.Items.Count / 3) > 0 then
    rgValue.Columns := rgValue.Columns + 1;
  *)
end;

procedure TfraEditObservationValue.QualifierComboResize(Sender: TObject);
begin
  if Sender is TComboBox then
  begin
    SetDropDownWidth(TComboBox(Sender));
  end;
end;

procedure TfraEditObservationValue.QualifierEnter(Sender: TObject);
begin
  {
  508
  blj 19 Mar 2009
  This is part 2 of the workaround described in NewEditObservationValueFrame up above.
  Since the screen reader won't read the text, we have to do it ourselves.
  }
  if not (Sender is TCombobox) then
    exit;
  GetScreenReader.Speak(TComboBox(Sender).Hint);
end;

procedure TfraEditObservationValue.SetAsComplex;
var
  tt: TTermType;
begin
  for tt := ttMethod to High(TTermType) do
    if fQualifier[tt] <> nil then
      fQualifier[tt].Visible := False;
end;

procedure TfraEditObservationValue.SetDefault(aTermType: TTermType; aTerm: ITerm);
var
  i: integer;
  ListTerm: ITerm;
begin
  if (fQualifier[aTermType] <> nil) and (aTerm <> nil) then
    begin
      fQualifier[aTermType].Visible := (fQualifier[aTermType].Items.Count > 0);
      for i := 0 to fQualifier[aTermType].Items.Count - 1 do
        if fQualifier[aTermType].Items.Objects[i].GetInterface(ITerm, ListTerm) then
          if ListTerm.getID = aTerm.getID then
            begin
              fQualifier[aTermType].ItemIndex := i;
              Exit;
            end;
      fQualifier[aTermType].ItemIndex := -1;
    end;
end;

procedure TfraEditObservationValue.SetMandatory(aTermType: TTermType; aTerm: ITerm);
begin
  if (fQualifier[aTermType] <> nil) and (aTerm <> nil) then
    begin
      SetDefault(aTermType, aTerm);
      fQualifier[aTermType].Style := csSimple;
      fQualifier[aTermType].Enabled := False;
      fQualifier[aTermType].Font.Style := [fsBold];
    end;
end;

procedure TfraEditObservationValue.SetNotAllowed(aTermType: TTermType);
begin
  if (fQualifier[aTermType] <> nil) then
    begin
      fQualifier[aTermType].ItemIndex := -1;
      fQualifier[aTermType].Visible := False;
    end;
end;

procedure TfraEditObservationValue.sbtnCommentsClick(Sender: TObject);
begin
  pnlComment.Visible := (not pnlComment.Visible);
  if pnlComment.Visible then
    Height := pnlComment.Top + pnlComment.Height + bvlBottom.Height
  else
    Height := pnlMain.Height + bvlBottom.Height;
  Invalidate;
end;

procedure TfraEditObservationValue.Validate(aErrorLog: TStringList);
begin
  //
end;

end.

