unit mFlowsheetTotals;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Grids,
  ExtCtrls,
  StdCtrls, VA508AccessibilityManager;

type
  TfraFlowsheetTotals = class(TForm)
    sg: TStringGrid;
    pnlTitle: TPanel;
    Shape1: TShape;
    Shape2: TShape;
    Shape3: TShape;
    A508Mgr: TVA508AccessibilityManager;
    lblTitle: TVA508StaticText;
    procedure sgDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
  private
    { Private declarations }
  public
    procedure ResizeTotals;
    { Public declarations }
  end;

function NewFlowsheetTotalsFrame(aOwner: TWinControl; aTop: integer = 0): TfraFlowsheetTotals;

implementation

uses
  VA508AccessibilityRouter;

{$R *.dfm}

function NewFlowsheetTotalsFrame(aOwner: TWinControl; aTop: integer = 0): TfraFlowsheetTotals;
begin
  Result := TfraFlowsheetTotals.Create(aOwner);
  with Result do
    begin
      Parent := aOwner;
      if ScreenReaderSystemActive then
      begin
        sg.DefaultDrawing := true;
        sg.OnDrawCell := nil;
        sg.GridLineWidth := 1;
      end;
      A508Mgr.AccessData.Add.Component := sg;
      DoubleBuffered := not ScreenReaderSystemActive;
      sg.DoubleBuffered := not ScreenReaderSystemActive;
      pnlTitle.DoubleBuffered := not ScreenReaderSystemActive;
      Top := aTop;
      Align := alTop;
      lblTitle.Font.Style := [fsBold];
      sg.Canvas.Font := Screen.IconFont;
      sg.DefaultRowHeight := sg.Canvas.TextHeight('|') + 4;
      sg.DefaultColWidth := 120;
    end;
end;

procedure TfraFlowsheetTotals.ResizeTotals;
begin
  Height :=
    pnlTitle.Height +
    (sg.RowCount * (sg.DefaultRowHeight + 1));
end;

procedure TfraFlowsheetTotals.sgDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with sg.Canvas do
    begin
      if (aCol = 0) or (aRow = 0) then
        begin
          Brush.Color := clInfoBk;
          Font.Style := [fsBold];
        end
      else if Frac(aRow / 2) = 0 then
        begin
          Brush.Color := cl3DLight;
          Font.Style := [];
        end
      else
        begin
          Brush.Color := clWindow;
          Font.Style := [];
        end;
      FillRect(Rect);

      if (aCol > 0) and (aRow > 0) then
        begin
          Pen.Color := cl3DLight;
          MoveTo(Rect.Right, Rect.Top);
          LineTo(Rect.Right, Rect.Bottom);
        end;

      {Everything beyond col 0 is right justified}
      if aCol = 0 then
        TextOut(Rect.Left + 2, Rect.Top + 2, sg.Cells[aCol, aRow])
      else
        TextOut(Rect.Right - TextWidth(sg.Cells[aCol, aRow]) - 2, Rect.Top + 2, sg.Cells[aCol, aRow]);
    end;
end;

end.

