unit mGridView;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  ActnList,
  ComCtrls,
  ImgList,
  ToolWin,
  iCliOGrid,
  oInterfaces,
  oCliO,
  oCliOTypes,
  mFlowsheetTotals,
  oCPTypes,
  uGlobalVars,
  Buttons,
  Menus,
  fImageList, System.Actions;

type
  TfraGridView = class(TForm, ICPView)
    ActionList1: TActionList;
    acDateRangeSetManually: TAction;
    acDateRangeExtendBack: TAction;
    acDateRangePanBack: TAction;
    acDateRangePanForward: TAction;
    acDateRangeExtendForward: TAction;
    acDateRangeToday: TAction;
    scbxFlowsheet: TScrollBox;
    acOnDateChange: TAction;
    acOnFrameClear: TAction;
    acActivatePage: TAction;
    acOnFrameRefresh: TAction;
    acSelectFlowsheet: TAction;
    acOnReloadList: TAction;
    pgctrl: TPageControl;
    tbshtGrids: TTabSheet;
    tbshtInput: TTabSheet;
    acOnCloseEdit: TAction;
    acDeactivatePage: TAction;
    Bevel1: TBevel;
    pmnView: TPopupMenu;
    AddPage2: TMenuItem;
    ClosePage3: TMenuItem;
    Refresh1: TMenuItem;
    mmFlowsheetMain: TMainMenu;
    FlowsheetTools1: TMenuItem;
    Refresh2: TMenuItem;
    AddPage1: TMenuItem;
    ClosePage1: TMenuItem;
    pnlGridCommands: TPanel;
    ckbxVerified: TCheckBox;
    ckbxCorrected: TCheckBox;
    ckbxLockViews: TCheckBox;
    Label1: TLabel;
    lblFlowsheetSelector: TLabel;
    cbxFlowsheet: TComboBox;
    acShowVerified: TAction;
    mmVerified: TMenuItem;
    N1: TMenuItem;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    tmrRefresh: TTimer;
    procedure acOnReloadListExecute(Sender: TObject);
    procedure acSelectFlowsheetExecute(Sender: TObject);
    procedure acOnFrameRefreshExecute(Sender: TObject);
    procedure acOnDateChangeExecute(Sender: TObject);
    procedure acOnCloseEditExecute(Sender: TObject);
    procedure acActivatePageExecute(Sender: TObject);
    procedure acDeactivatePageExecute(Sender: TObject);
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure ckbxLockViewsClick(Sender: TObject);
    procedure acReportExecute(Sender: TObject);
    procedure acShowVerifiedExecute(Sender: TObject);
    procedure tmrRefreshTimer(Sender: TObject);
    procedure cbxFlowsheetDropDown(Sender: TObject);
    procedure cbxFlowsheetCloseUp(Sender: TObject);
  protected
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  private
    fFlowsheet: ICPFlowsheet;
    fFlowsheetTotals: TfraFlowsheetTotals;
    fGridFrame: IGridFrame;
    fGrids: IInterfaceList;
    fViews: IInterfaceList;
    fGridTop: integer;

    FMenuItem: TMenuItem;

    //
    FNewViewID: string;

    // DRM - I16749714FY18 - 2017/10/24 - Simple mutex for the Refresh event.
    FRefreshing: boolean;
    // DRM ---

    function AddGrid(aView: ICPFlowsheetView; aTitle: string; aAllowClose: boolean = False): IGridFrame; overload;
    function AddGrid(aKardex: ICPKardexService; aAllowClose: boolean = False): IGridFrame; overload;

    {ICPView interface}
    procedure ViewInitialize; stdcall;
    procedure ViewActivate; stdcall;
    procedure ViewReload; stdcall;
    procedure ViewDeactivate; stdcall;
    function ViewMenu: TMenuItem; stdcall;
    function ViewHelpContext: Integer; stdcall;

    {IGridInterface callback methods}
    function InsertObservations(aGridView: IInterface): boolean; stdcall;
    function ClosedSupplemental(aGridView: IInterface): boolean; stdcall;
    function ClosedSupplementalObs(aItem: IInterface): boolean; stdcall;
    function SynchronizeGrids(aCallingGrid: IInterface; aTopRow: integer; aLeftCol: integer): boolean; stdcall;
    function GetItemProperties(aItem: IInterface): boolean; stdcall;
    function GetObsInViewTerm(aViewObs, aViewTerm: IInterface): boolean; stdcall;
    function GetTermIndex(aViewInterface: IInterface; aItem: IInterface): integer; stdcall;
    function GetObservations(aList: IInterfaceList): integer; stdcall;
    function GetObservationDisplayFont(aItem: IInterface; var aFont: TFont): boolean; stdcall;
    function GetObservationDisplayBackgroundColor(aItem: IInterface; var aColor: TColor): boolean; stdcall;
    function GetGridProperties(aItem: IInterface): boolean; stdcall;
    function GetGridItemFlag(aItem: IInterface): TGridFlag; stdcall;

    function GetKardexEvents(aList: IInterfaceList): integer; stdcall;
    function GetKardexActionIndex(aKardex: IInterface; aAction: IInterface): integer; stdcall;
    function GetKardexEventInAction(aEvent, aAction: IInterface): boolean; stdcall;

    function GetMaximumValue(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function GetMinimumValue(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function GetAverage(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function GetTotal(aList: IInterfaceList; aTerm: IInterface): string; stdcall;
    function GetCount(aList: IInterfaceList; aTerm: IInterface): integer; stdcall;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetFlowsheet(aFlowsheet: ICPFlowsheet);
  end;

function NewGridViewFrame(aOwner: TWinControl): TfraGridView;

implementation

uses
  fMain,
  DateUtils,
  fSelectDateRange,
  fObservationViewer,
  fSelectNewPage,
  mEditObservationBattery,
  mTaskView,
  fKardexViewer,
  fKardexAddEvent,
  mReportView,
  uCP508,
  u508Dialogs
  , System.UITypes // 2FA
  ;

{$R *.dfm}

function NewGridViewFrame(aOwner: TWinControl): TfraGridView;
begin
  Result := TfraGridView.Create(aOwner);
  with Result do
    begin
      Parent := aOwner;
      Align := alClient;

      TInterfaceList.Create.GetInterface(IInterfaceList, fGrids);
      TInterfaceList.Create.GetInterface(IInterfaceList, fViews);

      Result.DoubleBuffered := True;
      pnlGridCommands.DoubleBuffered := True;

      tbshtInput.TabVisible := False;
      tbshtGrids.TabVisible := False;
      pgctrl.ActivePage := tbshtGrids;
      pgctrl.DoubleBuffered := True;
      tbshtGrids.DoubleBuffered := True;
      tbshtInput.DoubleBuffered := True;

      lblFlowsheetSelector.Transparent := False;
      lblFlowsheetSelector.Font.Style := [fsBold];

      Visible := True;
    end;
end;

procedure TfraGridView.ViewActivate;
var
  i: integer;
  aFlowsheet: ICPFlowsheet;
begin
  if not Assigned(fFlowsheet) then
    for i := 0 to cbxFlowsheet.Items.Count - 1 do
      if cbxFlowsheet.Items.Objects[i].GetInterface(ICPFlowsheet, aFlowsheet) then
        if aFlowsheet.getID = MyUserOptions.getDefaultFlowsheetID then
        begin
          cbxFlowsheet.ItemIndex := i;
          aFlowsheet.QueryInterface(ICPFlowsheet, fFlowsheet);
          ViewReload;
        end;
  Visible := True;
  acSelectFlowsheet.Execute;
end;

function TfraGridView.ViewHelpContext: Integer;
begin
  result := HelpContext;
end;

procedure TfraGridView.ViewInitialize;
begin
  MyCPService.LoadList(cbxFlowsheet.Items, ciFlowsheet, True);
  // blj 15 August 2011: Timer intervals are in milliseconds.  Gotta multiply here. ;)
  tmrRefresh.Interval := MyUserOptions.getAutoRefreshInterval * 1000;
  tmrRefresh.Enabled :=  MyUserOptions.getAutoRefreshEnabled;
end;

procedure TfraGridView.ViewReload;
var
  i, j: integer;
  aPage: ICPFlowsheetPage;
  aView: ICPFlowsheetView;
  aTitle: string;
  aViews: IInterfaceList;
begin
  scbxFlowsheet.Visible := False;
  acDeactivatePage.Enabled := False;
  acActivatePage.Enabled := False;
  acOnFrameRefresh.Enabled := False;
  ckbxVerified.Enabled := False;
  ckbxCorrected.Enabled := False;
  ckbxLockViews.Checked := False;
  ckbxLockViews.Enabled := False;
  Application.ProcessMessages;

  {Clear out the old ones if there}
  (*
  for I := 0 to fGrids.Count - 1 do
    begin
      j := FMenuItem.IndexOf(IGridFrame(fGrids[I]).getMenu);
      if J > -1 then
        FMenuItem.Delete(j);
    end;
  *)
  {We now return you to your regularly scheduled program.}

  fGrids.Clear;
  //fViews.Clear;
  TInterfaceList.Create.GetInterface(IInterfaceList, aViews);

  {Free any left over grids}
  while scbxFlowsheet.ComponentCount > 0 do
    scbxFlowsheet.Components[0].Free;

  fFlowsheetTotals := nil; {If it existed, we cleaned it up above}


  if not Assigned(fFlowsheet) then
    Exit;

  if not Assigned(MyPatient) then
    Exit;

  {Check for views}
  MyCPService.LoadAllSupplementalViews(MyPatient.getDFN, MyStartDateTime, MyStopDateTime, aViews);

  {Go fetch a fresh copy of the pages and such}

  fFlowsheet.Load;

  fGridTop := 0; {Need this so that grids aligned top stay in order, updated in AddGrid}

  for j := 0 to fFlowsheet.PageCount - 1 do
    begin
      aPage := fFlowsheet.Page(j);
      aView := MyCPService.CPFlowsheetView(aPage.getFlowsheetViewID);

      if not Assigned(aView) then
        ShowMessage('Unable to find view id ' + aPage.getFlowsheetViewID);

      aView.Load;

      if not aView.getActive then
        MessageDlg(
          'Flowsheet Page ''' + aPage.getDisplayName + ''' was deactivated and cannot be loaded now.',
          mtInformation, [mbOk], 0)
      else
        case aPage.getPageType of
          ptMandatory:
            begin
              aTitle := aPage.getDisplayName;
              if aPage.getSpecialInstructions <> '' then
                aTitle := aTitle + #13 + aPage.getSpecialInstructions;
              AddGrid(aView, aTitle, False);
            end;
          ptOptional..ptSupplemental:
            begin
              for i := 0 to aViews.Count - 1 do
              begin
                aView := nil;
                if aViews[i].QueryInterface(ICPFlowsheetView, aView) = 0 then
                  if aView.getPageType = aPage.getPageType then
                    if aView.getID = aPage.getFlowsheetViewID then
                      if aView.getActivatedDateTime < MyStopDateTime then
                        begin
                          aView.Load;
                          acDeactivatePage.Enabled := true;

                          if aView.getSupplementalID <> FNewViewID then
                            AddGrid(aView, aPage.getDisplayName, False)
                          else
                            scbxFlowsheet.ScrollInView(AddGrid(aView, aPage.getDisplayName, False).getForm);
                        end;
              end;
            end;
        end;
      aView := nil;
    end;

  acActivatePage.Enabled := not MyUser.HasRole(ROLE_CLIO_READ_ONLY);
  acDeactivatePage.Enabled := not MyUser.HasRole(ROLE_CLIO_READ_ONLY);

  { DONE 1 -oBrian Juergensmeyer -cErrors : blj FIXME: This is where Kardex is blowing up if you view log items }
  {
  if MyPatient.getWardLocation <> '' then
    if MyCPParameters.getUseKardex(MyPatient.getWardLocation) then
      if fFlowsheet.getDisplayKardex then
        begin
          MyCPKardex.Load(MyPatient, MyStartDateTime, MyStopDateTime);
          AddGrid(MyCPKardex, False);
        end;
  }
  
  {Do we have totals to display?}
  if fFlowsheet.TotalCount > 0 then
    begin
      fFlowsheetTotals := NewFlowsheetTotalsFrame(scbxFlowsheet, fGridTop);
      with fFlowsheetTotals do
        begin
          fFlowsheetTotals.Top := fGridTop;
          fGridTop := fGridTop + fFlowsheetTotals.Height + 10;
          Visible := True;
          Align := alTop;
        end;
    end;

  acOnFrameRefresh.Enabled := True;
  acOnFrameRefresh.Execute;

  pnlGridCommands.Visible := true;
  scbxFlowsheet.Visible := True;
  ckbxVerified.Enabled := True;
  ckbxCorrected.Enabled := True;
  ckbxLockViews.Enabled := True;

  fViews.Clear;
  for i := 0 to aViews.Count - 1 do
    fViews.Add(aViews[i]);

  aViews.Clear;
  aViews := nil;
  Application.ProcessMessages;
end;

procedure TfraGridView.ViewDeactivate;
begin
  Visible := False;
end;

procedure TfraGridView.acActivatePageExecute(Sender: TObject);
begin

  fNewViewID := ActivatePage(fFlowsheet);
  if fNewViewID <> '' then
    SetFlowsheet(fFlowsheet);
end;

procedure TfraGridView.acDeactivatePageExecute(Sender: TObject);
begin
  DeactivatePage(fGrids);
  SetFlowsheet(fFlowsheet);
end;

procedure TfraGridView.acOnCloseEditExecute(Sender: TObject);
begin
  //tbarGridView.Visible := True;
  pnlGridCommands.Visible := True;
  pgctrl.ActivePage := tbshtGrids;
  acOnFrameRefresh.Execute;
  scbxFlowsheet.VertScrollBar.Position := scbxFlowsheet.Tag;
end;

procedure TfraGridView.acOnDateChangeExecute(Sender: TObject);
var
  i: integer;
  aGridFrame: IGridFrame;
begin
  if Assigned(fFlowsheet) then
    begin
      acOnReloadList.Execute; {Get new data}
      for i := 0 to fGrids.Count - 1 do
        if fGrids[i].QueryInterface(IGridFrame, aGridFrame) = 0 then
          begin
            aGridFrame.BeginUpdate;
            aGridFrame.setStartDate(MyStartDateTime);
            aGridFrame.setStopDate(MyStopDateTime);
            aGridFrame.EndUpdate;
            aGridFrame.Refresh;
          end;
    end;
end;

procedure TfraGridView.acOnFrameRefreshExecute(Sender: TObject);
{This method is called externally and internally to keep this frame updated}
var
  i: integer;
  aGridFrame: IGridFrame;
begin
  // DRM - I16749714FY18 - 2017/10/24 - Simple mutex for the Refresh event.
  if FRefreshing then
    Exit;
  FRefreshing := True;
  try
  // DRM ---
  if Assigned(fFlowsheet) then
    try
      tmrRefresh.Enabled := False; {Stop this thing from getting in the way}
      acOnReloadList.Execute;
      for i := 0 to fGrids.Count - 1 do
        if fGrids[i].QueryInterface(IGridFrame, aGridFrame) = 0 then
          aGridFrame.Refresh;

      if fFlowsheet.TotalCount > 0 then
        begin
          fFlowsheetTotals.sg.RowCount := 2;
          for i := 0 to fFlowsheet.TotalCount - 1 do
            begin
              if i > 0 then {Add a space for subsequent totals}
                fFlowsheetTotals.sg.RowCount := fFlowsheetTotals.sg.RowCount + 2;
              try
                fFlowsheet.Total(i).Execute(MyCliOList);
                fFlowsheet.Total(i).AppendToGrid(fFlowsheetTotals.sg);
              except
                on E: Exception do
                  MessageDlg('Error encountered executing ' +
                    fFlowsheet.Total(i).getDisplayName + #13 +
                    E.Message, mtError, [mbOk], 0);
              end;
            end;
          fFlowsheetTotals.ResizeTotals;
        end;
    finally
      tmrRefresh.Enabled := MyUserOptions.getAutoRefreshEnabled;
    end;
  {Return to the grid by default}
  pgctrl.ActivePage := tbshtGrids;
  // DRM - I16749714FY18 - 2017/10/24 - Simple mutex for the Refresh event.
  finally
    FRefreshing := False;
  end;
  // DRM ---
end;

procedure TfraGridView.acOnReloadListExecute(Sender: TObject);
var
  aTmpList: ICLiOList;
begin
  aTmpList := MyCliOLoader.CreateNewList;
  MyCliOList.Clear;
  try
    if ckbxVerified.Checked then
      begin
        MyCliOLoader.LoadObservationsByStatus(aTmpList, MyPatient.getDFN, MyStartDateTime, MyStopDateTime, [osVerified]);
        aTmpList.CopyTo(MyCliOList);
      end;

    if ckbxCorrected.Checked then
      begin
        MyCliOLoader.LoadObservationsByStatus(aTmpList, MyPatient.getDFN, MyStartDateTime, MyStopDateTime, [osCorrected]);
        aTmpList.CopyTo(MyCliOList);
      end;

  except
    on E: Exception do
      MessageDlg('Error reloading observation list.' + #13 + E.Message, mtError, [mbOk], 0);
  end;
end;

procedure TfraGridView.acReportExecute(Sender: TObject);
begin
  if cbxFlowsheet.ItemIndex > -1 then
    begin
      FlowsheetReport(cbxFlowsheet.ItemIndex);
    end
  else
    begin
      SHowMessage('Select a flowsheet and try again...');
    end;
end;

procedure TfraGridView.acSelectFlowsheetExecute(Sender: TObject);
begin
  if cbxFlowsheet.ItemIndex > -1 then
    if cbxFlowsheet.Items.Objects[cbxFlowsheet.ItemIndex].GetInterface(ICPFlowsheet, fFlowsheet) then
      SetFlowsheet(fFlowsheet);
end;

procedure TfraGridView.acShowVerifiedExecute(Sender: TObject);
begin
  ckbxVerified.Checked := (not ckbxVerified.Checked);
  mmVerified.Checked := (not mmVerified.Checked);
end;

{----------------------- Internal private methods -----------------------------}

function TfraGridView.AddGrid(aView: ICPFlowsheetView; aTitle: string; aAllowClose: boolean = False): IGridFrame;
var
  aGridFrame: IGridFrame;
  aViewTerm: ICPFlowsheetViewTerm;
  i: integer;
  aMethod: string;
  aPosition: string;
  aLocation: string;
  aProduct: string;
begin
  if aView.getSupplementalID <> '' then
    if (aView.getDeactivatedDateTime > 0) and (aView.getDeactivatedDateTime < Now) then
      aGridFrame := NewGridFrame(scbxFlowsheet, aView, fGridTop, False)
    else
      aGridFrame := NewGridFrame(scbxFlowsheet, aView, fGridTop)
  else
    aGridFrame := NewGridFrame(scbxFlowsheet, aView, fGridTop);

  {Assign the callback methods}
  aGridFrame.OnGridFetchTotal := GetTotal;
  aGridFrame.OnGridFetchAverage := GetAverage;
  aGridFrame.OnGridFetchCount := GetCount;
  aGridFrame.OnGridFetchMinValue := GetMinimumValue;
  aGridFrame.OnGridFetchMaxValue := GetMaximumValue;

  aGridFrame.OnGridFetchData := GetObservations;
  aGridFrame.OnGridFindTerm := GetTermIndex;
  aGridFrame.OnGridClickItem := GetItemProperties;

  if aView.getSupplementalID <> '' then
    if (aView.getDeactivatedDateTime > 0) and (aView.getDeactivatedDateTime < Now) then
      begin
        aGridFrame.OnGridInsertData := ClosedSupplemental;
        aGridFrame.OnGridClickItem := ClosedSupplementalObs;
      end
    else
      aGridFrame.OnGridInsertData := InsertObservations
  else
    aGridFrame.OnGridInsertData := InsertObservations;

  aGridFrame.OnGridIsItemInTerm := GetObsInViewTerm;
  aGridFrame.OnGridGetItemFont := GetObservationDisplayFont;
  aGridFrame.OnGridGetItemColor := GetObservationDisplayBackgroundColor;
  aGridFrame.OnGridProperties := GetGridProperties;
  aGridFrame.OnGridSynchNeeded := SynchronizeGrids;
  aGridFrame.OnGridGetItemFlag := GetGridItemFlag;

  {Add the terms}
  for i := 0 to aView.getViewTermCount - 1 do
    begin
      aViewTerm := aView.getViewTermByIndex(i);
      aGridFrame.AddTerm(aViewTerm as IGridTerm);

      {Do we need totals rows?}
      if not aGridFrame.getIncludeTotals then
        aGridFrame.setIncludeTotals(aViewTerm.getDisplayTotal);
      if not aGridFrame.getIncludeCounts then
        aGridFrame.setIncludeCounts(aViewTerm.getDisplayCount);
      if not aGridFrame.getIncludeAverages then
        aGridFrame.setIncludeAverages(aViewTerm.getDisplayAverage);
      if not aGridFrame.getIncludeMinValue then
        aGridFrame.setIncludeMinValue(aViewTerm.getDisplayMinValue);
      if not aGridFrame.getIncludeMaxValue then
        aGridFrame.setIncludeMaxValue(aViewTerm.getDisplayMaxValue);
    end;

  {Setup display attributes}
  aGridFrame.setInterval(aView.getDisplayInterval);
  aGridFrame.setTermsOnXAxis(aView.getViewStyle = vsTerminology);
  aGridFrame.setAllowPivot(aView.getAllowPivot);

  case aView.getPageType of
    ptMandatory:
      begin
        aGridFrame.setTitle(aTitle);
      end;
    ptOptional:
      begin
        aGridFrame.setTitle('OPTIONAL view: ' + aTitle + ' - ' + aView.getDisplayName +
          ': ' + aView.getComment);
      end;
    ptSupplemental:
      begin
        if assigned(aView.getDefaultMethod) then
          aMethod := aView.getDefaultMethod.getDisplayName
        else
          aMethod := '';

        if assigned(aView.getDefaultPosition) then
          aPosition := aView.getDefaultPosition.getDisplayName
        else
          aPosition := '';

        if assigned(aView.getDefaultLocation) then
          aLocation := aView.getDefaultLocation.getDisplayName
        else
          aLocation := '';

        if assigned(aView.getDefaultProduct) then
          aProduct := aView.getDefaultProduct.getDisplayName
        else
          aProduct := '';

        aGridFrame.setSupplementalMethod(aMethod);
        aGridFrame.setSupplementalPosition(aPosition);
        aGridFrame.setSupplementalLocation(aLocation);
        aGridFrame.setSupplementalProduct(aProduct);

        if aMethod <> '' then
          aMethod := 'METHOD: ' + aMethod;

        if aPosition <> '' then
          aPosition := ' POSITION: ' + aPosition;

        if aLocation <> '' then
          aLocation := ' LOCATION: ' + aLocation;

        if aProduct <> '' then
          aProduct := ' PRODUCT: ' + aProduct;

        aGridFrame.setTitle('SUPPLEMENTAL view: ' + aTitle + ' ' + aView.getSupplementalTitle +
        aMethod + aPosition + aLocation + aProduct);
        //
      end;
    else
      raise Exception.Create('Invalid page type in AddGrid()');
  end;


  (*
  if aView.getPageType = ptMandatory then
    aGridFrame.setTitle(aView.getDisplayName)
  else
    aGridFrame.setTitle(aView.getSupplementalTitle);
  *)

  {Add them to the interface lists}
  aGridFrame.setStartDate(MyStartDateTime);
  aGridFrame.setStopDate(MyStopDateTime);

  fGrids.Add(aGridFrame);
  fViews.Add(aView);
  fGridTop := fGridTop + aGridFrame.getHeight + 10;

  if aGridFrame.getHeight > 200 then
    fGridTop := fGridTop + aGridFrame.getHeight + 10
  else
    fGridTop := fGridTop + 220;

  {
  FMenuItem.Add(aGridFrame.getMenu);
  }

  aGridFrame.QueryInterface(IGridFrame, result);
end;

function TfraGridView.AddGrid(aKardex: ICPKardexService; aAllowClose: boolean = False): IGridFrame;
var
  aGridFrame: IGridFrame;
  aKardexAction: IGridTerm;
  i: integer;
begin
  aGridFrame := NewGridFrame(scbxFlowsheet, aKardex, fGridTop);

  aGridFrame.OnGridFetchData := GetKardexEvents;
  aGridFrame.OnGridFindTerm := GetKardexActionIndex;
  aGridFrame.OnGridClickItem := GetItemProperties;
  aGridFrame.OnGridInsertData := InsertObservations;
  aGridFrame.OnGridIsItemInTerm := GetKardexEventInAction;

  aGridFrame.OnGridGetItemFont := GetObservationDisplayFont;
  aGridFrame.OnGridGetItemColor := GetObservationDisplayBackgroundColor;
  aGridFrame.OnGridProperties := GetGridProperties;
  aGridFrame.OnGridSynchNeeded := SynchronizeGrids;
  aGridFrame.OnGridGetItemFlag := GetGridItemFlag;

  {Add the actions}
  for i := 0 to aKardex.getActionCount - 1 do
    begin
      aKardex.getAction(i).QueryInterface(IGridTerm, aKardexAction);
      aGridFrame.AddTerm(aKardexAction);
    end;

  {Setup display attributes}
  aGridFrame.setInterval(480);
  aGridFrame.setTermsOnXAxis(False);
  aGridFrame.setAllowPivot(False);
  aGridFrame.setTitle('Kardex');
  aGridFrame.setInsertCaption('Add Event ...', 'Document a PRN, One-Time, Now or Stat event.');

  aGridFrame.setStartDate(MyStartDateTime);
  aGridFrame.setStopDate(MyStopDateTime);

  fGrids.Add(aGridFrame);
  fGridTop := fGridTop + aGridFrame.getHeight + 10;
  FMenuItem.Add(AGridFrame.getMenu);
  result := aGridFrame;
end;

procedure TfraGridView.cbxFlowsheetCloseUp(Sender: TObject);
begin
  tmrRefresh.Enabled := (cbxFlowsheet.ItemIndex > -1) and MyUserOptions.getAutoRefreshEnabled; {Just causes havoc on selecting an item}
end;

procedure TfraGridView.cbxFlowsheetDropDown(Sender: TObject);
begin
  tmrRefresh.Enabled := False; {Just causes havoc on selecting an item}
end;

procedure TfraGridView.ckbxLockViewsClick(Sender: TObject);
var
  aView: IGridFrame;
  i: integer;
  aInterval: integer;
begin
  aInterval := 0;
  if ckbxLockViews.Checked then
    begin
      for i := 0 to fGrids.Count - 1 do
        if fGrids[i].QueryInterface(IGridFrame, aView) = 0 then
          if i = 0 then
            begin
              aInterval := aView.getInterval;
              aView.LockGrid(aInterval, gsTerminologyList);
              aView.setExpanded(True);
            end
          else
            begin
              aView.LockGrid(aInterval, gsTerminologyList);
              aView.setExpanded(True);
            end
    end
  else
    for i := 0 to fGrids.Count - 1 do
      if fGrids[i].QueryInterface(IGridFrame, aView) = 0 then
        aView.UnLockGrid;
end;

function TfraGridView.ClosedSupplemental(aGridView: IInterface): boolean;
begin
  MessageDlg508('This is a closed supplemental page and can''t be added to.', mtInformation, [mbOk], 0);
  Result := True;
end;

function TfraGridView.ClosedSupplementalObs(aItem: IInterface): boolean;
begin
  MessageDlg508('Observations in closed supplemental views may not be viewed on the flowsheet.', mtError, mbOkCancel, 0);
  result := false;
end;

constructor TfraGridView.Create(AOwner: TComponent);
var
  Item: TMenuItem;
begin
  inherited;

  FMenuItem := TMenuItem.Create(Self);
  FMenuItem.Caption := 'Flowsheet';

  Item := TMenuItem.Create(FMenuItem);
  Item.Action := acOnFrameRefresh;
  FMenuItem.Add(Item);

  Item := TMenuItem.Create(FMenuItem);
  Item.Action := acActivatePage;
  FMenuItem.Add(Item);

  Item := TMenuItem.Create(FMenuItem);
  Item.Action := acDeactivatePage;
  FMenuItem.Add(Item);

  TMenuItem.Create(FMenuItem).Caption := '-';
  with TMenuItem.Create(FMenuItem) do
    begin
      Action := acShowVerified;
      Checked := true;
    end;

  TMenuItem.Create(FMenuItem).Caption := '-';

end;

function TfraGridView.GetMaximumValue(aList: IInterfaceList; aTerm: IInterface): string;
begin
  Result := MyCliOService.getMaximumValue(aList, aTerm);
end;

function TfraGridView.GetMinimumValue(aList: IInterfaceList; aTerm: IInterface): string;
begin
  Result := MyCliOService.getMinimumValue(aList, aTerm);
end;

function TfraGridView.GetTotal(aList: IInterfaceList; aTerm: IInterface): string;
begin
  Result := MyCliOService.getTotal(aList, aTerm);
end;

function TfraGridView.GetCount(aList: IInterfaceList; aTerm: IInterface): integer;
begin
  Result := MyCliOService.getCount(aList, aTerm);
end;

function TfraGridView.GetAverage(aList: IInterfaceList; aTerm: IInterface): string;
begin
  Result := MyCliOService.getAverage(aList, aTerm);
end;

function TfraGridView.GetGridItemFlag(aItem: IInterface): TGridFlag;
var
  aObservation: IObservation;
  aEvent: ICPKardexEvent;
begin
  result := gfNone;
  if aItem.QueryInterface(IObservation, aObservation) = 0 then
    begin
      if aObservation.getComment <> '' then
        Result := gfInfo
      else
        Result := gfNone;
    end
  else
    if aItem.QueryInterface(ICPKardexEvent, aEvent) = 0 then
      begin
        if aEvent.getComment <> '' then
          Result := gfInfo
        else
          Result := gfNone;
      end
    else
      ShowMessage('Grid has received a non-grid interface item.');
end;

function TfraGridView.GetGridProperties(aItem: IInterface): boolean;
var
  aView: ICPFlowsheetView;
begin
  if aItem.QueryInterface(ICPFlowsheetView, aView) = 0 then
    MessageDlg(
      'Grid View Properties:' + #13 + #13 +
      'View ID: ' + aView.getID + #13 +
      'Page Type: ' + CP_FLOWSHEET_PAGE_TYPE[aView.getPageType],
      mtInformation, [mbOk], 0)
  else
    MessageDlg('Ooops, something called GetGridProperties but it wasn''t a flowsheet view', mtError, [mbOk], 0);
  result := assigned(aView);
end;

function TfraGridView.GetKardexActionIndex(aKardex, aAction: IInterface): integer;
var
  i: integer;
  aCPKardex: ICPKardexService;
  aCPAction: ICPKardexAction;
begin
  if aKardex.QueryInterface(ICPKardexService, aCPKardex) = 0 then
    if aAction.QueryInterface(ICPKardexAction, aCPAction) = 0 then
      for i := 0 to aCPKardex.getActionCount - 1 do
        if aCPKardex.getAction(i).getID = aCPAction.getID then
          begin
            Result := i;
            Exit;
          end;
  Result := -1;
end;

function TfraGridView.GetKardexEventInAction(aEvent, aAction: IInterface): boolean;
var
  aCPEvent: ICPKardexEvent;
  aCPAction: ICPKardexAction;
begin
  result := false;
  if aAction.QueryInterface(ICPKardexAction, aCPAction) = 0 then
    if aEvent.QueryInterface(ICPKardexEvent, aCPEvent) = 0 then
      Result := (aCPAction.getID = aCPEvent.getActionID);
end;

function TfraGridView.GetKardexEvents(aList: IInterfaceList): integer;
var
  i, j: integer;
begin
  aList.Clear;
  for i := 0 to MyCPKardex.getActionCount - 1 do
    for j := 0 to MyCPKardex.getAction(i).getEventCount - 1 do
      aList.Add(MyCPKardex.getAction(i).getEvent(j));
  Result := MyCPKardex.getActionCount;
end;

function TfraGridView.GetObservationDisplayBackgroundColor(aItem: IInterface; var aColor: TColor): boolean;
var
  aObservation: IObservation;
begin
  if aItem.QueryInterface(IObservation, aObservation) = 0 then
    if MyUserOptions.getFlowsheetItemColor(aObservation.getRange) <> clWindow then
      begin
        aColor := MyUserOptions.getFlowsheetItemColor(aObservation.getRange);
        Result := True;
      end
    else
      Result := False
  else
    Result := False;
end;

function TfraGridView.GetObservationDisplayFont(aItem: IInterface; var aFont: TFont): boolean;
var
  aObservation: IObservation;
  aEvent: ICPKardexEvent;
begin
  if aItem.QueryInterface(IObservation, aObservation) = 0 then
    if aObservation.getStatus = osCorrected then
      aFont.Style := [fsStrikeOut]
    else
      aFont.Assign(MyUserOptions.getFlowsheetItemFont(aObservation.getRange))
  else
    if aItem.QueryInterface(ICPKardexEvent, aEvent) = 0 then
      if aEvent.getStatus = esPending then
        aFont.Style := [fsBold]
      else
        aFont.Style := [];
  result := true;
end;

function TfraGridView.GetItemProperties(aItem: IInterface): boolean;
var
  aObservation: IObservation;
  aEvent: ICPKardexEvent;
begin
  if aItem.QueryInterface(IObservation, aObservation) = 0 then
    begin
      DisplayObservation(aObservation, MyCliOLoader);
      acOnFrameRefresh.Execute;
    end
  else
    if aItem.QueryInterface(ICPKardexEvent, aEvent) = 0 then
      begin
        ViewKardexEvent(MyCPKardex, aEvent);
        MyCPKardex.Load(MyPatient, MyStartDateTime, MyStopDateTime);
        fGridFrame.Refresh;
      end
    else
      ShowMessage('Observation controller has received a non-observation interface');
  result := true;
end;

function TfraGridView.GetObservations(aList: IInterfaceList): integer;
var
  i: integer;
begin
  aList.Clear;
  for i := 0 to MyCliOList.ObservationCount - 1 do
    aList.Add(MyCliOList.Observation(i));
  result := MyClioList.ObservationCount;
end;

function TfraGridView.GetObsInViewTerm(aViewObs, aViewTerm: IInterface): boolean;
var
  aObs: IObservation;
  aTerm: ICPFlowsheetViewTerm;
begin
  Result := False;
  if aViewObs.QueryInterface(IObservation, aObs) = 0 then
    if aViewTerm.QueryInterface(ICPFlowsheetViewTerm, aTerm) = 0 then
      if aObs.IsPartOfSet(aTerm.getDefaultSetID) then
        case aObs.getStatus of

          osVerified:
            if ckbxVerified.Checked then
              Result := aTerm.ContainsObservation(aObs);

          osCorrected:
            if ckbxCorrected.Checked then
              Result := aTerm.ContainsObservation(aObs);

        end;
end;

function TfraGridView.GetTermIndex(aViewInterface: IInterface; aItem: IInterface): integer;
var
  aView: ICPFlowsheetView;
  aObs: IObservation;
begin
  Result := -1; {Just in case it isn't found below :) }
  if aViewInterface.QueryInterface(ICPFlowsheetView, aView) = 0 then
    if aItem.QueryInterface(IObservation, aObs) = 0 then
      if aObs.IsPartOfSet(aView.getDefaultSetID) then
        case aObs.getStatus of
          osVerified:
            if ckbxVerified.Checked then
              Result := aView.GetViewTermIndex(aItem);
          osCorrected:
            if ckbxCorrected.Checked then
              Result := aView.GetViewTermIndex(aItem);
        end;
end;

function TfraGridView.InsertObservations(aGridView: IInterface): boolean;
var
  aView: ICPFlowsheetView;
  aKardex: ICPKardexService;
  aAction: ICPKardexAction;
  i: integer;
begin
  if aGridView.QueryInterface(ICPFlowsheetView, aView) = 0 then
    begin
      try
        {Check to see if a supplemental has expired}
        if (aView.getSupplementalID <> '') and (aView.getDeactivatedByID <> '') and (aView.getDeactivatedDateTime < Now) then
          raise Exception.Create('The active timeframe for this supplemental page has expired and it can no longer be updated.');

        {Check for leftovers from previous input}
        while tbshtInput.ComponentCount > 0 do
          TfraEditObservationBattery(tbshtInput.Components[0]).Free;

        with NewEditObservationBatteryFrame(tbshtInput, aView, MyCliOFiler) do
          begin
            setOnCancelEdit(acOnCloseEditExecute);
            Visible := true;
          end;

        scbxFlowsheet.Tag := scbxFlowsheet.VertScrollBar.Position;
        //tbarGridView.Visible := False;
        pnlGridCommands.Visible := False;
        pgctrl.ActivePage := tbshtInput;
        Result := True;
      except
        on E: Exception do
          begin
            Result := False;
            MessageDlg508('Error opening edit frame.' + #13 + E.Message, mtError, [mbOk], 0);
          end;
      end;
    end
  else
    if aGridView.QueryInterface(ICPKardexService, aKardex) = 0 then
      with TfrmKardexAddEvent.Create(application) do
        try
          for i := 0 to MyCPKardex.getActionCount - 1 do
            if MyCPKardex.getAction(i).QueryInterface(ICPKardexAction, aAction) = 0 then
              if aAction.getActive then
                case aAction.getScheduleType of
                  stContinuous:
                    begin
                      if aAction.getSchedulePRNType <> prnNot then
                        aAction.AddToList(lbxActions.Items);
                    end;
                  stPRN:
                    begin
                      if aAction.getPRNNTE > 0 then
                        if aAction.getEventCount < aAction.getPRNNTE then
                          aAction.AddToList(lbxActions.Items)
                        else
                      else
                        aAction.AddToList(lbxActions.Items);
                    end;
                else
                  aAction.AddToList(lbxActions.Items);
                end;
          if ShowModal = mrOk then
            begin
              MyCPKardex.CompleteEvent(getKardexAction, MyUser, getDateTime, getComment);
              ViewReload;
            end;
          result := true;
        finally
          Free;
        end
    else
      begin
        MessageDlg508('Error, Unable to get the interface for this grid', mtError, [mbOk], 0);
        result := false;
      end;
end;

procedure TfraGridView.SetFlowsheet(aFlowsheet: ICPFlowsheet);
var
  I: Integer;
  AGridFrame: IGridFrame;
begin
  if fFlowsheet <> aFlowsheet then
    aFlowsheet.QueryInterface(ICPFlowsheet, fFlowsheet);

  {Get the pointer from the controller so it updates it}
  ckbxLockViews.Checked := False;
  {Now call reload}
  ViewReload;

  {
  Now that the view is reloaded, we need to walk through and redo the menus.
  }
  for I := 0 to fGrids.Count - 1 do
  begin
    if fGrids[I].QueryInterface(IGridFrame, AGridFrame) = 0 then
      FMenuItem.Add(AGridFrame.getMenu);
  end;
    
end;

function TfraGridView.SynchronizeGrids(aCallingGrid: IInterface; aTopRow, aLeftCol: integer): boolean;
var
  i: integer;
  aGridFrame: IGridFrame;
begin
  //ShowMessage('synch grid called out!');
  if ckbxLockViews.Checked then
    for i := 0 to fGrids.Count - 1 do
      if fGrids[i].QueryInterface(IGridFrame, aGridFrame) = 0 then
        aGridFrame.SynchGrid(aCallingGrid, aTopRow, aLeftCol);
  result := true;
end;

procedure TfraGridView.tmrRefreshTimer(Sender: TObject);
begin
  tmrRefresh.Enabled := False;
  Self.Enabled := False;
  {Now decide if this is a good idea}
  try
    if MyUserOptions.getAutoRefreshEnabled then
      if Self.Visible then
        if pgctrl.ActivePage = tbshtGrids then
          if Assigned(MyPatient) then
            if not cbxFlowsheet.DroppedDown then {Creates havoc and unpredictable stuff :( }
              if cbxFlowsheet.ItemIndex > -1 then
                  acOnFrameRefresh.Execute;
              {
              else
                Application.MainForm.Caption := 'No flowsheet selected ' + DateTimeToStr(Now)
            else
              Application.MainForm.Caption := 'User trying to select a flowsheet ' + DateTimeToStr(Now)
          else
            Application.MainForm.Caption := 'No patient selected ' + DateTimeToStr(Now)
        else
          Application.MainForm.Caption := 'Not on the grid page ' + DateTimeToStr(Now)
      else
        Application.MainForm.Caption := 'Grid frame is invisible ' + DateTimeToStr(Now)
    else
      Application.MainForm.Caption := 'User doesn''t want auto-refresh' + DateTimeToStr(Now);
    }
  finally
    Self.Enabled := True;
    tmrRefresh.Enabled := MyUserOptions.getAutoRefreshEnabled;
  end;
end;

function TfraGridView._AddRef: Integer;
begin
  Result := -1;
end;

function TfraGridView._Release: Integer;
begin
  Result := -1;
end;

procedure TfraGridView.TreeView1Change(Sender: TObject; Node: TTreeNode);
begin
  {Free any left over grids}
  (*
  while scbxFlowsheet.ComponentCount > 0 do
    scbxFlowsheet.Components[0].Free;
  fFlowsheetTotals := nil; {If it existed, we cleaned it up above}

  {Clear out the old ones if there}
  fGrids.Clear;
  fViews.Clear;

  if Assigned(Node.Data) then
    if TObject(Node.Data).GetInterface(ICPFlowsheetView, aView) then
      AddGrid(aView);
  *)
end;

destructor TfraGridView.Destroy;
begin
  fFlowsheet := nil;
  //ViewReload;
  //FreeAndNil(FMenuItem);
  inherited;
end;

function TfraGridView.ViewMenu: TMenuItem; stdcall;
begin
  Result := FMenuItem;
end;

end.

