unit mLogView;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ComCtrls,
  ToolWin,
  ExtCtrls,
  ImgList,
  ActnList,
  oCliO,
  oCliOTypes,
  oInterfaces,
  uGlobalVars,
  StdCtrls,
  Buttons,
  Menus,
  VA508AccessibilityManager,
  fImageList;

type
  TfraLogView = class(TForm, ICPView)
    lvObservations: TListView;
    ActionList1: TActionList;
    acVerify: TAction;
    acPurge: TAction;
    acSelAll: TAction;
    acDeSelAll: TAction;
    acRefresh: TAction;
    tBarGridDateSelector: TToolBar;
    acUnverified: TAction;
    acVerified: TAction;
    acPurged: TAction;
    acCorrected: TAction;
    acRescinded: TAction;
    tbtnPurge: TToolButton;
    tbtnVerify: TToolButton;
    acPrint: TAction;
    acRefreshDateTime: TAction;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    acChangeDateTimeRange: TAction;
    acPrintSelected: TAction;
    tbtnRescind: TToolButton;
    acRescind: TAction;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton10: TToolButton;
    tvLog: TTreeView;
    Splitter1: TSplitter;
    Bevel1: TBevel;
    Bevel2: TBevel;
    pmnView: TPopupMenu;
    Refresh2: TMenuItem;
    SelectAll2: TMenuItem;
    ClearSelection2: TMenuItem;
    N3: TMenuItem;
    Purge2: TMenuItem;
    Verify2: TMenuItem;
    Rescind2: TMenuItem;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    acRecover: TAction;
    ToolButton7: TToolButton;
    acEdit: TAction;
    ToolButton8: TToolButton;
    procedure acPrintExecute(Sender: TObject);
    procedure acVerifyExecute(Sender: TObject);
    procedure acPurgeExecute(Sender: TObject);
    procedure acSelAllExecute(Sender: TObject);
    procedure acDeSelAllExecute(Sender: TObject);
    procedure acRefreshExecute(Sender: TObject);
    procedure lvObservationsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure lvObservationsColumnClick(Sender: TObject; Column: TListColumn);
    procedure acPrintSelectedExecute(Sender: TObject);
    procedure acRescindExecute(Sender: TObject);
    procedure tvLogChange(Sender: TObject; Node: TTreeNode);
    procedure tvLogGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure ActionList1Update(Action: TBasicAction; var Handled: Boolean);
    procedure acRecoverExecute(Sender: TObject);
    procedure acEditExecute(Sender: TObject);
  protected
    fCliOList: ICliOList;
    fCliOLogFile: IInterfaceList;
    fSortColumn: integer;
    fReverseSort: boolean;
    fStatusNodes: array[TObservationRecordStatus] of TTreeNode;
    fCurrentLog: TObservationRecordStatus;
    FMenuItem: TMenuItem;

    procedure PrintObservations(aSelectedOnly: boolean);

    {ICPView interface}
    procedure ViewInitialize; stdcall;
    procedure ViewActivate; stdcall;
    procedure ViewReload; stdcall;
    procedure ViewDeactivate; stdcall;
    function ViewMenu: TMenuItem; stdcall;
    function ViewHelpContext: Integer; stdcall;
    
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

function NewLogViewFrame(aOwner: TWinControl): TfraLogView;

implementation

uses
  Math,
  MFunStr,
  DateUtils,
  fSelectDateRange,
  Printers,
  u508Dialogs,
  fObservationViewer;

{$R *.dfm}

function NewLogViewFrame(aOwner: TWinControl): TfraLogView;
begin
  Result := TfraLogView.Create(aOwner);
  with Result do
    begin
      Parent := aOwner;
      Align := alClient;
      fSortColumn := 0;
      fCliOList := MyCliOLoader.CreateNewList;
      TInterfaceList.Create.GetInterface(IInterfaceList, fCliOLogFile);
      tbtnRescind.Visible := MyUser.HasRole('MD ADMINISTRATOR');
      if MyUser.HasRole('MD ADMINISTRATOR') then
        with lvObservations.Columns.Add do
          begin
            Caption := 'Source ID';
            Width := 120;
            Alignment := taLeftJustify;
          end;
      Visible := False;
    end;
end;

{***************************** ICPView methods ********************************}

procedure TfraLogView.ViewActivate;
begin
  ViewReload;
  Visible := True;
end;

procedure TfraLogView.ViewDeactivate;
var
  aStatus: TObservationRecordStatus;
begin
  for aStatus := Low(TObservationRecordStatus) to High(TObservationRecordStatus) do
    fStatusNodes[aStatus].DeleteChildren;
  lvObservations.Items.Clear;

  Visible := False;
end;

function TfraLogView.ViewHelpContext: Integer;
begin
  result := HelpContext;
end;

procedure TfraLogView.ViewInitialize;
var
  aStatus: TObservationRecordStatus;
  aRootNode: TTreeNode;
begin
  tvLog.Items.Clear;
  aRootNode := tvLog.Items.AddChild(nil, 'Observation Log');
  for aStatus := Low(TObservationRecordStatus) to High(TObservationRecordStatus) do
    fStatusNodes[aStatus] := tvLog.Items.AddChild(aRootNode, OBSERVATION_STATUS[aStatus]);
end;

procedure TfraLogView.ViewReload;
var
  i: integer;
  aCliOLogEntry: ICliOLogEntry;
  aStatus: TObservationRecordStatus;
begin
  lvObservations.Items.Clear;

  for aStatus := Low(TObservationRecordStatus) to High(TObservationRecordStatus) do
    fStatusNodes[aStatus].DeleteChildren;

  tvLog.Selected := tvLog.TopItem;
  tvLog.TopItem.Expand(True);

  if Assigned(MyPatient) then
    begin
      MyCliOService.LoadLogEntries(fCliOLogFile, MyPatient.getDFN, MyStartDateTime, MyStopDateTime);
      for i := 0 to fCliOLogFile.Count - 1 do
        if fCliOLogFile[i].QueryInterface(ICliOLogEntry, aCliOLogEntry) = 0 then
          aCliOLogEntry.AddToTreeView(fStatusNodes[aCliOLogEntry.getStatus]);
    end;

  if Printers.Printer <> nil then
    begin
      acPrintSelected.Hint := Format('Print selected items to printer ''%s''', [Printer.Printers[Printer.PrinterIndex]]);
      acPrint.Hint := Format('Print all items to printer ''%s''', [Printer.Printers[Printer.PrinterIndex]]);
    end
  else
    begin
      acPrintSelected.Hint := 'No Printer Defined';
      acPrint.Hint := 'No Printer Defined';
    end;
end;

procedure TfraLogView.tvLogChange(Sender: TObject; Node: TTreeNode);
var
  aCliOLogEntry: ICliOLogEntry;
  aObservation: IObservation;
  aStatus: TObservationRecordStatus;

  procedure LoadObservations(aPtID: string; aStartDateTime, aStopDateTime: TDateTime; aStatus: TObservationRecordStatus);
  begin
    fCurrentLog := aStatus;
    MyCliOLoader.LoadObservationsByStatus(fCliOList, aPtID, aStartDateTime, aStopDateTime, [aStatus]);
    aObservation := fCliOList.FirstObservation;
    while Assigned(aObservation) do
      try
        with aObservation.AddToListView(lvObservations) do
          begin
            SubItems.Add(FormatDateTime('YYYY-MM-DD hh:mm:ss', aObservation.getObservedDateTime));
            SubItems.Add(aObservation.getObservedByInitials);
            SubItems.Add(aObservation.getStatusValue);
            SubItems.Add(aObservation.getDisplayValue);
            SubItems.Add(StringReplace(aObservation.getSourceDataItemID,'\u094','^',[rfReplaceAll]));
            ImageIndex := -1;
          end;
      finally
        aObservation := fCliOList.NextObservation(aObservation);
      end;
  end;

begin
  if lvObservations.Items.Count > 0 then
    lvObservations.Items.Clear;

  if Assigned(Node.Data) then
    begin
      if TObject(Node.Data).GetInterface(ICliOLogEntry, aCliOLogEntry) then
        begin
          LoadObservations(
            aCliOLogEntry.getPatientID,
            aCliOLogEntry.getDateTime,
            aCliOLogEntry.getDateTime,
            aCliOLogEntry.getStatus);
          fSortColumn := -1;
          lvObservationsColumnClick(Sender, lvObservations.Columns[0]);
        end;
    end
  else {Is it a status node?}
    for aStatus := Low(TObservationRecordStatus) to High(TObservationRecordStatus) do
      if fStatusNodes[aStatus] = Node then
        begin
          LoadObservations(
            MyPatient.getDFN,
            MyStartDateTime,
            MyStopDateTime,
            aStatus);
          fSortColumn := -1;
          lvObservationsColumnClick(Sender, lvObservations.Columns[1]);
        end;
end;

procedure TfraLogView.tvLogGetImageIndex(Sender: TObject; Node: TTreeNode);
begin
  if Assigned(Node.Data) then
    begin
      Node.ImageIndex := LV_PROCEDURE_SML;
      Node.SelectedIndex := LV_PROCEDURE_SML;
    end
  else
    if Node.Expanded or Node.Selected then
      begin
        Node.ImageIndex := BTN_FLDR_OPEN;
        Node.SelectedIndex := BTN_FLDR_OPEN;
      end
    else
      begin
        Node.ImageIndex := BTN_FLDR_CLOSED;
        Node.SelectedIndex := BTN_FLDR_CLOSED;
      end;
end;

procedure TfraLogView.acVerifyExecute(Sender: TObject);
var
  i: integer;
  aObs: IObservation;
begin
  lvObservations.Items.BeginUpdate;
  try
    for i := lvObservations.Items.Count - 1 downto 0 do
      if lvObservations.Items[i].Selected then
        if TObject(lvObservations.Items[i].Data).GetInterface(IObservation, aObs) then
          begin
            MyCliOFiler.UpdateObservationAsVerified(aObs, MyUser, 'Verified with ' + Application.Title);
            lvObservations.Items.Delete(i);
          end;
  finally
    lvObservations.Items.EndUpdate;
  end;
end;

constructor TfraLogView.Create(AOwner: TComponent);
var
  Item: TMenuItem;
begin
  inherited;

  FMenuItem := TMenuItem.Create(nil);
  FMenuItem.Caption := 'Log';

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acRefresh;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acSelAll;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acDeSelAll;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Caption := '-';

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acPurge;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acVerify;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acRescind;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acEdit;
end;

destructor TfraLogView.Destroy;
begin
  FreeAndNil(FMenuItem);
  inherited;
end;

procedure TfraLogView.lvObservationsColumnClick(Sender: TObject; Column: TListColumn);
var
  i: integer;
begin
  if fSortColumn = Column.Index then {Clicking same column - reverse the sort}
    fReverseSort := not fReverseSort
  else
    fReverseSort := False;

  fSortColumn := Column.Index;
  for i := 0 to lvObservations.Columns.Count - 1 do
    lvObservations.Columns[i].ImageIndex := -1;
  if fReverseSort then
    lvObservations.Columns[fSortColumn].ImageIndex := LV_SORT_REVERSE
  else
    lvObservations.Columns[fSortColumn].ImageIndex := LV_SORT_FORWARD;
  lvObservations.AlphaSort;
end;

procedure TfraLogView.lvObservationsCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
  aObs1: IObservation;
  aObs2: IObservation;
begin
  if Assigned(Item1.Data) and Assigned(Item2.Data) then
    if TObject(Item1.Data).GetInterface(IObservation, aObs1) then
      if TObject(Item2.Data).GetInterface(IObservation, aObs2) then
        begin
          case fSortColumn of
            0: {Sorts by name and then by Date/Time}
              begin
                Compare := AnsiCompareStr(Item1.Caption, Item2.Caption);
                if Compare = 0 then
                  Compare := CompareDateTime(aObs1.getObservedDateTime, aObs2.getObservedDateTime);
              end;
            1: {Sorts by Date/Time and then Name}
              begin
                Compare := CompareDateTime(aObs1.getObservedDateTime, aObs2.getObservedDateTime);
                if Compare = 0 then
                  Compare := AnsiCompareStr(Item1.Caption, Item2.Caption);
              end;
            2: {Sorts by Initials and then by Date/Time}
              begin
                Compare := AnsiCompareStr(Item1.SubItems[1], Item2.SubItems[1]);
                if Compare = 0 then
                  Compare := CompareDateTime(aObs1.getObservedDateTime, aObs2.getObservedDateTime);
              end;
            3: {Sorts by status (redundant) and then by Date/Time}
              begin
                Compare := AnsiCompareStr(Item1.SubItems[2], Item2.SubItems[2]);
                if Compare = 0 then
                  Compare := CompareDateTime(aObs1.getObservedDateTime, aObs2.getObservedDateTime);
              end;
            4:
              begin
                Compare := CompareValue(StrToFloatDef(Piece(Item1.SubItems[3], ' '), 0), StrToFloatDef(Piece(Item2.SubItems[3], ' '), 0));
                if Compare = 0 then
                  Compare := AnsiCompareStr(Item1.Caption, Item2.Caption);
              end;
          end;
          if Compare = 0 then
            Compare := AnsiCompareStr(Item1.Caption, Item2.Caption);
        end;

  if fReverseSort then
    Compare := Compare * -1;
end;

procedure TfraLogView.acPurgeExecute(Sender: TObject);
var
  i: integer;
  aObs: IObservation;
  aCmt: string;
begin
  if MessageDlg508(
    'Warning: You are about to discard these observations' + #13 +
    'from this patients record.' + #13 + #13 +
    'These observations will be permanently removed after' + #13 +
    'the grace period defined by the system administrator' + #13 +
    'is reached.' + #13 +
    'Are you sure this is what you want to do?',
    mtWarning, mbYesNo, 0) = mrYes then
    //if InputQuery('Observation Purge', 'Comment:', aCmt) then
    //  if aCmt <> '' then
    try
      aCmt := 'Purged ' + DateTimeToStr(Now);
      lvObservations.Items.BeginUpdate;
      for i := lvObservations.Items.Count - 1 downto 0 do
        if lvObservations.Items[i].Selected then
          if TObject(lvObservations.Items[i].Data).GetInterface(IObservation, aObs) then
            begin
              MyCliOFiler.UpdateObservationAsPurged(aObs, MyUser, aCmt);
              lvObservations.Items.Delete(i);
            end;
    finally
      lvObservations.Items.EndUpdate;
    end
    //  else
    //    MessageDlg508('Sorry, I can''t purge these without a comment.', mtInformation, [mbOk], 0);
end;

procedure TfraLogView.acRescindExecute(Sender: TObject);
var
  i: integer;
  aObs: IObservation;
  aCmt: string;
begin
  if MessageDlg508(
    'Warning: You are about to rescind these observations' + #13 +
    'from this patients record.' + #13 + #13 +
    'These observations will be permanently hidden' + #13 +
    'from view for non-priveledged users.' + #13 + #13 +
    'Are you sure this is what you want to do?',
    mtWarning, mbYesNo, 0) = mrYes then
    if InputQuery('Observation Rescission', 'Comment:', aCmt) then
      try
        lvObservations.Items.BeginUpdate;
        for i := lvObservations.Items.Count - 1 downto 0 do
          if lvObservations.Items[i].Selected then
            if TObject(lvObservations.Items[i].Data).GetInterface(IObservation, aObs) then
              begin
                MyCliOFiler.UpdateObservationAsRescinded(aObs, MyUser, aCmt);
                lvObservations.Items.Delete(i);
              end;
      finally
        lvObservations.Items.EndUpdate;
      end;
end;

procedure TfraLogView.PrintObservations(aSelectedOnly: boolean);
begin
  with TPrinterSetupDialog.Create(Application) do
    try
      if Execute then
        ShowMessage('Time to print this puppy')
      else
        ShowMessage('No hard copy for you');
    finally
      free;
    end;
end;

procedure TfraLogView.acEditExecute(Sender: TObject);
var
  aItem: TListItem;
  aObservation: IObservation;
begin
  aItem := lvObservations.Selected;
  if not Assigned(aItem) then
    exit;

  if TObject(aItem.Data).GetInterface(IObservation, aObservation) then
    begin
      DisplayObservation(aObservation, MyCliOLoader);
    end;
end;

procedure TfraLogView.acPrintExecute(Sender: TObject);
begin
  (*
  with oCliO.CreateNewReportGenerator do
    begin
      SetTitle(Format('%s Observations', [OBSERVATION_STATUS[fLogFile]]));
      SetPatient(MyPatient);
      AddSubTitle(
        'Date: ' + FormatDateTime('MM/DD/YYYY HH:MM:SS', fStartDateTime) +
        ' - ' + FormatDateTime('MM/DD/YYYY HH:MM:SS', fStopDateTime));
      PrintListView(lvObservations);
    end;
  *)
end;

procedure TfraLogView.acPrintSelectedExecute(Sender: TObject);
begin
  MessageDlg508('Under Construction', mtInformation, [mbOk], 0);
end;

procedure TfraLogView.acSelAllExecute(Sender: TObject);
begin
  lvObservations.SelectAll;
end;

procedure TfraLogView.ActionList1Update(Action: TBasicAction; var Handled: Boolean);
var
  aStatus: TObservationRecordStatus;
  aObservation: IObservation;
  aStudent: Boolean;
  aReadOnly: Boolean;
begin
  aStatus := osRescinded; {Nothing works with this one :)}
  aStudent := MyUser.HasRole('MD TRAINEE');
  aReadOnly :=  MyUser.HasRole(ROLE_CLIO_READ_ONLY);

  if lvObservations.Items.Count > 0 then
  begin
    {
      We need to find out if we have an individual observation selected.
    }


    if lvObservations.SelCount > 0 then
      TObject(lvObservations.Selected.Data).GetInterface(IObservation, aObservation)
    else
      TObject(lvObservations.Items[0].Data).GetInterface(IObservation, aObservation);

    {
      No specific observation was selected.  We need to just get generic observations
    }

    aStatus := aObservation.getstatus;
    aReadOnly := aReadOnly or aObservation.isReadOnly;
  end;

  acVerify.Enabled := (lvObservations.SelCount > 0)
    and (aStatus = osUnverified)
    and (not aStudent)
    and (not aReadOnly);
  acPurge.Enabled := (lvObservations.SelCount > 0)
    and (aStatus = osUnverified)
    and (not aStudent)
    and (not aReadOnly);
  acSelAll.Enabled := (lvObservations.Items.Count > 0);
  acDeSelAll.Enabled := (lvObservations.SelCount > 0);
  acRescind.Enabled := (lvObservations.SelCount > 0)
    and (aStatus = osVerified)
    and (not aStudent)
    and (not aReadOnly);
  acEdit.Enabled := (lvObservations.SelCount = 1)
    and (aStatus = osVerified)
    and (not aStudent)
    and (not aReadOnly);
  (*
  acPrint.Enabled := (lvObservations.Items.Count > 0);
  acPrintSelected.Enabled := (lvObservations.SelCount > 0);
  *)
  acRecover.Enabled := (lvObservations.SelCount > 0)
    and (aStatus = osPurged)
    and (not aStudent)
    and (not aReadOnly);
  Handled := True;
end;

procedure TfraLogView.acDeSelAllExecute(Sender: TObject);
var
  i: integer;
begin
  for i := 0 to lvObservations.Items.Count - 1 do
    lvObservations.Items[i].Selected := False;
end;

procedure TfraLogView.acRecoverExecute(Sender: TObject);
var
  i: integer;
  aObs: IObservation;
begin
  if MessageDlg508(
    'Warning: You are about to recover these observations' + #13 +
    'from the purge log for this patients record.' + #13 + #13 +
    'Are you sure this is what you want to do?',
    mtWarning, mbYesNo, 0) = mrYes then
    try
      lvObservations.Items.BeginUpdate;
      for i := lvObservations.Items.Count - 1 downto 0 do
        if lvObservations.Items[i].Selected then
          if TObject(lvObservations.Items[i].Data).GetInterface(IObservation, aObs) then
            begin
              MyCliOFiler.UpdateObservationAsUnverified(aObs, MyUser);
              lvObservations.Items.Delete(i);
            end;
    finally
      lvObservations.Items.EndUpdate;
    end
end;

procedure TfraLogView.acRefreshExecute(Sender: TObject);
begin
  ViewReload;
end;

function TfraLogView.ViewMenu: TMenuItem; stdcall;
begin
  Result := FMenuItem;
end;

end.

