unit mReportView;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  ComCtrls,
  ToolWin,
  oCliO,
  oInterfaces,
  oCliOTypes,
  oCPTypes,
  StdCtrls,
  XMLDoc,
  XMLIntf,
  uGlobalVars,
  ActnList,
  ImgList,
  Buttons,
  Menus,
  VA508AccessibilityManager,
  fImageList, System.Actions;
// iCP_View, Menus // zzzzzzandria 2008-07-09

type
  TfraReportView = class(TForm, ICPView)
    ActionList1: TActionList;
    acRefresh: TAction;
    PrinterSetupDialog1: TPrinterSetupDialog;
    acRebuild: TAction;
    acPrint: TAction;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    Bevel1: TBevel;
    lblReportID: TLabel;
    cbxReport: TComboBox;
    acViewSource: TAction;
    acSetReport: TAction;
    acWriteNote: TAction;
    Bevel2: TBevel;
    memReport: TMemo;
    pnlNavigator: TPanel;
    bbtnFirst: TBitBtn;
    bbtnPrev: TBitBtn;
    bbtnNext: TBitBtn;
    bbtnLast: TBitBtn;
    acPageFirst: TAction;
    acPageLast: TAction;
    acPagePrev: TAction;
    acPageNext: TAction;
    acUpdateNavigator: TAction;
    pmnView: TPopupMenu;
    Refresh2: TMenuItem;
    Print2: TMenuItem;
    WriteNote2: TMenuItem;
    N2: TMenuItem;
    First2: TMenuItem;
    Prev2: TMenuItem;
    Next2: TMenuItem;
    Last2: TMenuItem;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    ckbxComments: TCheckBox;
    ToolButton3: TToolButton;
    procedure acRefreshExecute(Sender: TObject);
    procedure acPrintExecute(Sender: TObject);
    procedure acWriteNoteExecute(Sender: TObject);
    procedure acPageFirstExecute(Sender: TObject);
    procedure acUpdateNavigatorExecute(Sender: TObject);
    procedure acPageNextExecute(Sender: TObject);
    procedure acPagePrevExecute(Sender: TObject);
    procedure acPageLastExecute(Sender: TObject);
  private
    fCliOReport: ICliOReport;
    fCurrentPage: integer;
    fListOfReportItems: IInterfaceList;

    FMenuItem: TMenuItem;

    function getFlowsheet: ICPFlowsheet;

    procedure ViewInitialize; stdcall;
    procedure ViewActivate; stdcall;
    procedure ViewReload; stdcall;
    procedure ViewDeactivate; stdcall;
    function ViewMenu: TMenuItem; stdcall;
    function ViewHelpContext: Integer; stdcall;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  end;

function NewReportViewFrame(aOwner: TWinControl): TfraReportView;
procedure FlowsheetReport(aFlowsheetID: Integer);

implementation

uses
  Printers,
  fNoteWriter,
  u508Dialogs;

{$R *.dfm}

var
  fReportDialog: TForm;

function NewReportViewFrame(aOwner: TWinControl): TfraReportView;
begin
  Result := TfraReportView.Create(aOwner);
  with Result do
    begin
      Parent := aOwner;
      Align := alClient;
      Visible := False;
    end;
end;

procedure FlowsheetReport(aFlowsheetID: Integer);
var
  fReportView: TFraReportView;
  i: Integer;
begin
  fReportView := nil;
  if not Assigned(fReportDialog) then
    begin
      Application.CreateForm(TForm, fReportDialog);
      with fReportDialog do
        begin
          Caption := 'Flowsheet report';
          width := 800;
          height := 600;
          color := clWindow;
          position := poScreenCenter;
          fReportView := NewReportViewFrame(fReportDialog);
          fReportView.lblReportID.Visible := False;
          fReportView.cbxReport.Visible := False;
          fReportView.Toolbutton5.Visible := False;
          fReportView.Parent := fReportDialog;
          fReportView.Align := alClient;
          //Menu :=  fReportView.ViewMenu;
          fReportView.ViewInitialize;
          fReportView.Show;
        end;
    end
  else
    begin
      for i := 0 to fReportDialog.ComponentCount - 1 do
        begin
          if fReportDialog.Components[i] is TfraReportView then
            begin
              fReportView := TfraReportView(fReportDialog.Components[i]);
              break;
            end;
        end;
    end;

  if Assigned(fReportView) then
    begin
      fReportView.cbxReport.ItemIndex := aFlowsheetID;
      fReportDialog.Caption := '"' + fReportView.cbxReport.Text + '" Flowsheet Report';
      fReportView.ViewActivate;
    end;
  fReportDialog.ShowModal;

  if Assigned(fReportView) then
    fReportView.ViewDeactivate;
end;

////////////////////////////////////////////////////////////////////////////////

{ TfraReportView }

constructor TfraReportView.Create(aOwner: TComponent);
var
  Item: TMenuItem;
begin
  inherited Create(aOwner);
  lblReportID.Transparent := False;
  lblReportID.Font.Style := [fsBold];
  TInterfaceList.Create.GetInterface(IInterfaceList, fListOfReportItems);

  FMenuItem := TMenuItem.Create(nil);
  FMenuItem.Caption := 'Reports';

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acRefresh;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acPrint;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acWriteNote;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Caption := '-';

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acPageFirst;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acPagePrev;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acPageNext;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acPageLast;
end;

destructor TfraReportView.Destroy;
begin
  FreeAndNil(FMenuItem);

  fListOfReportItems.Clear;
  inherited;
end;

procedure TfraReportView.ViewActivate;
begin
  ViewReload;
  Visible := True;
end;

procedure TfraReportView.ViewDeactivate;
begin
  Visible := False;
end;

function TfraReportView.ViewHelpContext: Integer;
begin
  result := HelpContext;
end;

procedure TfraReportView.ViewInitialize;
begin
  MyCPService.LoadList(cbxReport.Items, ciFlowsheet, True);
  fCliOReport := MyCliOService.getNewReport(MyCliOLoader);
end;

procedure TfraReportView.ViewReload;
var
  aFlowsheet: ICPFlowsheet;
  aPage: ICPFlowsheetPage;
  aView: ICPFlowsheetView;
  aPtSuppViews: IInterfaceList;
  i, j: integer;
begin
  pnlNavigator.Caption := '';
  acPageFirst.Enabled := False;
  acPagePrev.Enabled := False;
  acPageNext.Enabled := False;
  acPageLast.Enabled := False;
  acPrint.Enabled := False;
  acWriteNote.Enabled := False;
  acRefresh.Enabled := False;

  if MyPatient = nil then
    begin
      memReport.Lines.Clear;
      memReport.Lines.Add('No Patient');
      Exit;
    end;

  aFlowsheet := getFlowsheet;

  if aFlowsheet = nil then
    begin
      memReport.Lines.Clear;
      memReport.Lines.Add('No flowsheet');
      Exit;
    end;

  try
    memReport.Lines.BeginUpdate;
    fListOfReportItems.Clear;
    aFlowsheet.Load;
    fCliOReport.setTitle(aFlowsheet.getDisplayName);
    {Check for pt sepcific supplemental views}
    TInterfaceList.Create.GetInterface(IInterfaceList, aPtSuppViews);
    //MyObservationSource.LoadActiveSupplementalViews(MyPatient.getDFN, aPtSuppViews);
    //MyCPService.LoadActiveSupplementalViews(MyPatient.getDFN, aPtSuppViews);
    MyCPService.LoadAllSupplementalViews(MyPatient.getDFN, MyStartDateTime, MyStopDateTime, aPtSuppViews);
    for j := 0 to aFlowsheet.PageCount - 1 do
      begin
        aPage := aFlowsheet.Page(j);
        case aPage.getPageType of
          ptMandatory:
            begin
              aView := MyCPService.CPFlowsheetView(aPage.getFlowsheetViewID);
              aView.Load;
              fListOfReportItems.Add(aView);
            end;
          ptOptional..ptSupplemental:
            begin
              for i := 0 to aPtSuppViews.Count - 1 do
                if aPtSuppViews[i].QueryInterface(ICPFlowsheetView, aView) = 0 then
                  if aView.getPageType = aPage.getPageType then
                    if aView.getID = aPage.getFlowsheetViewID then
                      begin
                        aView.Load;
                        fListOfReportItems.Add(aView);
                      end;
            end;
        end;
      end;

    if aFlowsheet.TotalCount > 0 then
      begin
        fListOfReportItems.Add(fCliOReport.getSectionBreak('F L O W S H E E T   T O T A L S  ' +
          FormatDateTime('mmm dd, yyyy HH:nn:ss', MyStartDateTime) + ' - ' +
          FormatDateTime('mmm dd, yyyy HH:nn:ss', MyStopDateTime), '=', taCenter));
        for j := 0 to aFlowsheet.TotalCount - 1 do
          fListOfReportItems.Add(aFlowsheet.Total(j));
      end;

    {
    if MyPatient.getWardLocation <> '' then
      begin
        if MyCPParameters.getUseKardex(MyPatient.getWardLocation) then
          if aFlowsheet.getDisplayKardex then
            fListOfReportItems.Add(MyCPKardex);

        acWriteNote.Enabled :=
          MyCPParameters.getUseTIUNoteWriter(MyPatient.getWardLocation) and
          (aFlowsheet.getDefaultTIUNoteID <> '');
      end;
    }
    fCliOReport.Refresh(MyStartDateTime, MyStopDateTime, MyPatient, fListOfReportItems, ckbxComments.Checked);
    acPageFirst.Enabled := True;
    acPageFirst.Execute;
    acRefresh.Enabled := True;
  finally
    aPtSuppViews.Clear;
    memReport.Lines.EndUpdate;
  end;
end;

function TfraReportView.getFlowsheet: ICPFlowsheet;
begin
  if cbxReport.ItemIndex > -1 then
    cbxReport.Items.Objects[cbxReport.ItemIndex].GetInterface(ICPFlowsheet, Result)
  else
    Result := nil;
end;

procedure TfraReportView.acWriteNoteExecute(Sender: TObject);
var
  aDocument: ICPTIUDocument;
  aText: TStringList;
begin
  if getFlowsheet <> nil then
    if getFlowsheet.getDefaultTIUNoteID <> '' then
      try
        aText := TStringList.Create;
        aDocument := NewCPTIUDocument(MyPatientSource.DBConnection, MyUser);
        aDocument.setPatient(MyPatient);
        aDocument.setTitleID(getFlowsheet.getDefaultTIUNoteID);
        aDocument.setTitle(getFlowsheet.getDefaultTIUNote);
        fCliOReport.getText(aText);
        WriteTIUDocument(aDocument, aText);
        aText.Clear;
        FreeAndNil(aText);
      except
        on E: Exception do
          begin
            MessageDlg508('Error writing note.' + #13 + e.Message, mtError, [mbOk], 0);
            aText.Clear;
            FreeAndNil(aText);
          end;
      end;
end;

procedure TfraReportView.acRefreshExecute(Sender: TObject);
begin
  ViewReload;
end;

procedure TfraReportView.acUpdateNavigatorExecute(Sender: TObject);
begin
  if getFlowsheet <> nil then
    begin
      pnlNavigator.Caption := Format('Page %d of %d', [fCurrentPage, fCliOReport.getPageCount]);
      acPageFirst.Enabled := (fCurrentPage > 1);
      acPagePrev.Enabled := (fCurrentPage > 1);
      acPageNext.Enabled := (fCurrentPage < fCliOReport.getPageCount);
      acPageLast.Enabled := (fCurrentPage < fCliOReport.getPageCount);
      acPrint.Enabled := True;
      acWriteNote.Enabled := (getFlowsheet.getDefaultTIUNoteID <> '') and not
        (MyUser.HasRole(ROLE_CLIO_READ_ONLY));
      acPrint.Enabled := not MyUser.HasRole(ROLE_CLIO_READ_ONLY);
    end;
end;

procedure TfraReportView.acPageFirstExecute(Sender: TObject);
begin
  try
    memReport.Lines.BeginUpdate;
    fCurrentPage := 1;
    fCliOReport.getPage(fCurrentPage, memReport.Lines);
    acUpdateNavigator.Execute;
  finally
    memReport.Lines.EndUpdate;
  end;
end;

procedure TfraReportView.acPagePrevExecute(Sender: TObject);
begin
  try
    memReport.Lines.BeginUpdate;
    dec(fCurrentPage);
    fCliOReport.getPage(fCurrentPage, memReport.Lines);
    acUpdateNavigator.Execute;
  finally
    memReport.Lines.EndUpdate;
  end;
end;

procedure TfraReportView.acPageNextExecute(Sender: TObject);
begin
  try
    memReport.Lines.BeginUpdate;
    inc(fCurrentPage);
    fCliOReport.getPage(fCurrentPage, memReport.Lines);
    acUpdateNavigator.Execute;
  finally
    memReport.Lines.EndUpdate;
  end;
end;

procedure TfraReportView.acPageLastExecute(Sender: TObject);
begin
  try
    memReport.Lines.BeginUpdate;
    fCurrentPage := fCliOReport.getPageCount;
    fCliOReport.getPage(fCurrentPage, memReport.Lines);
    acUpdateNavigator.Execute;
  finally
    memReport.Lines.EndUpdate;
  end;
end;

procedure TfraReportView.acPrintExecute(Sender: TObject);
var
  aReport: ICliOReport;
  aPage: integer;
  aBorderRect: TRect;
  aPageRect: TRect;
begin
  with TPrintDialog.Create(Self) do
    try
      if Execute then
        try
          Printer.Canvas.Font := memReport.Font;
          Printer.Canvas.MoveTo(0, 0); {Required to get the new font to apply}

          aReport := MyCliOService.getNewReport(MyCliOLoader);
          aReport.setTitle(getFlowsheet.getDisplayName);

          aBorderRect := Rect(0, 0, Printer.PageWidth, Printer.PageHeight);

          aPageRect := Rect(
            trunc(Printer.PageWidth * 0.02),
            trunc(Printer.PageHeight * 0.02),
            Printer.PageWidth - trunc(Printer.PageWidth * 0.02),
            Printer.PageHeight - trunc(Printer.PageHeight * 0.02));

          aReport.setMargins(
            trunc((aPageRect.Right - aPageRect.Left) / Printer.Canvas.TextWidth('X')) - 4,
            trunc((aPageRect.Bottom - aPageRect.Top) / Printer.Canvas.TextHeight('X')) - 4);

          {
          ShowMessage(
            Format('%s %d,%d,%d,%d', ['PrinterCanvas', 0, 0, Printer.PageWidth, Printer.PageHeight]) + #13 +
            Format('%s %d,%d,%d,%d', ['aBorderRect', aBorderRect.Left, aBorderRect.Top, aBorderRect.Right, aBorderRect.Bottom]) + #13 +
            Format('%s %d,%d,%d,%d', ['aPageRect', aPageRect.Left, aPageRect.Top, aPageRect.Right, aPageRect.Bottom]) + #13 +
            Format('%s %d  %d  %dx%d', ['Report Margins - TextWidth of X and W:', Printer.Canvas.TextWidth('X'), Printer.Canvas.TextWidth('W'),
            trunc((aPageRect.Right - aPageRect.Left) / Printer.Canvas.TextWidth('X')),
              trunc((aPageRect.Bottom - aPageRect.Top) / Printer.Canvas.TextHeight('X'))]));
          }

          aReport.Refresh(MyStartDateTime, MyStopDateTime, MyPatient, fListOfReportItems, ckbxComments.Checked);

          Printer.Title := 'CP Flowsheets - ' + getFlowsheet.getDisplayName;

          Printer.BeginDoc;

          for aPage := 0 to aReport.getPageCount - 1 do
            begin
              aReport.PrintPage(aPage + 1, Printer.Canvas, aPageRect);
              if aPage < (aReport.getPageCount - 1) then
                Printer.NewPage;
            end;
        finally
          Printer.EndDoc;
        end
    finally
      Free;
    end;
end;

function TfraReportView.ViewMenu: TMenuItem; stdcall;
begin
  Result := FMenuItem;
end;

end.

