unit mTaskView;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ComCtrls,
  ToolWin,
  ExtCtrls,
  ActnList,
  uGlobalVars,
  ImgList,
  oCliOTypes,
  oInterfaces,
  oCPTypes,
  Menus,
  VA508AccessibilityManager,
  fImageList;

type
  TfraTaskView = class(TForm, ICPView)
    lvKardex: TListView;
    ActionList1: TActionList;
    tBarGridDateSelector: TToolBar;
    ToolButton4: TToolButton;
    tbtnPurge: TToolButton;
    ToolButton3: TToolButton;
    tbtnRescind: TToolButton;
    Bevel2: TBevel;
    acRefresh: TAction;
    acNew: TAction;
    acRenew: TAction;
    acDiscontinue: TAction;
    acHold: TAction;
    acUnHold: TAction;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    Bevel1: TBevel;
    pmnView: TPopupMenu;
    Refresh2: TMenuItem;
    New2: TMenuItem;
    Renew2: TMenuItem;
    Discontinue2: TMenuItem;
    Hold3: TMenuItem;
    UnHold1: TMenuItem;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    procedure acRefreshExecute(Sender: TObject);
    procedure acDiscontinueExecute(Sender: TObject);
    procedure lvKardexSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure acRenewExecute(Sender: TObject);
    procedure acNewExecute(Sender: TObject);
    procedure acHoldExecute(Sender: TObject);
    procedure acUnHoldExecute(Sender: TObject);
  private
    FMenuItem: TMenuItem;
    procedure ViewInitialize; stdcall;
    procedure ViewActivate; stdcall;
    procedure ViewReload; stdcall;
    procedure ViewDeactivate; stdcall;
    function ViewMenu: TMenuItem; stdcall;
    function ViewHelpContext: integer; stdcall;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

function NewTaskViewFrame(aOwner: TWinControl): TfraTaskView;

implementation

uses
  DateUtils,
  fComment,
  fKardexOrder,
  fKardexHold,
  fSelectDate,
  u508Dialogs;

{$R *.dfm}

function NewTaskViewFrame(aOwner: TWinControl): TfraTaskView;
begin
  Result := TfraTaskView.Create(aOwner);
  with Result do
    begin
      Parent := aOwner;
      Align := alClient;
      Visible := False;
    end;
end;

{ TfraTaskView }

procedure TfraTaskView.acDiscontinueExecute(Sender: TObject);
var
  i: integer;
  aAction: ICPKardexAction;
  aComment: string;
  aPrompt: string;
begin
  if lvKardex.SelCount = 1 then
    aPrompt := 'Enter the reason for discontinuing this action:'
  else
    aPrompt := 'Enter the reason for discontinuing these actions:';

  if GetComment(aComment, 'Comment required to discontinue Kardex Action', aPrompt) then
    begin
      for i := 0 to lvKardex.Items.Count - 1 do
        with lvKardex.Items[i] do
          if Selected then
            if TObject(Data).GetInterface(ICPKardexAction, aAction) then
              if aAction.getStatus = asActive then
                MyCPKardex.DiscontinueAction(aAction, MyUser, aComment)
              else
                MessageDlg508(
                  Format('Action ''%s'' is not active and cannot be discontinued.', [aAction.getCareAction.getTerm]),
                  mtInformation, [mbOk], 0);
      ViewReload;
    end;
end;

procedure TfraTaskView.acHoldExecute(Sender: TObject);
var
  i: Integer;
//  j: integer;
  aAction: ICPKardexAction;
//  aEvent: ICPKardexEvent;
  aHoldDate: TDateTime;
  aComment: string;
begin
  aHoldDate := Now;
  aComment := '';
  for i := 0 to lvKardex.Items.Count - 1 do
    if lvKardex.Items[i].Selected then
      if TObject(lvKardex.Items[i].Data).GetInterface(ICPKardexAction, aAction) then
        if aAction.getStatus = asActive then
          if getKardexHoldDateAndComment(aAction, aHoldDate, aComment) then
            MyCPKardex.HoldAction(aAction, MyUser, aHoldDate, aAction.getStopDateTime, aComment);
  acRefresh.Execute;
end;

procedure TfraTaskView.acNewExecute(Sender: TObject);
var
  Hour, Min, Sec, MSec: Word;
begin
  with TfrmAddEditKardexItem.Create(Self) do
    try
      MyTerminology.LoadList(cbxCareAction.Items, ttTask);
      MyCPService.LoadList(cbxSchedule.Items, ciSchedule, True);
      DecodeTime(Now, Hour, Min, Sec, MSec);
      Min := trunc(Min / 15) * 15;
      dtpStartDate.DateTime := trunc(Now);
      dtpStartTime.DateTime := trunc(Now) + EncodeTime(Hour, Min, 0, 0);
      udDuration.Position := 14;
      memSpecialInstructions.Text := '';
      ShowModal;
    finally
      free;
    end;
  acRefresh.Execute;
end;

procedure TfraTaskView.acRefreshExecute(Sender: TObject);
begin
  ViewReload;
end;

procedure TfraTaskView.acRenewExecute(Sender: TObject);
var
  aAction: ICPKardexAction;
  aNewDate: TDateTime;
//  aComment: string;
begin
  if lvKardex.Selected <> nil then
    if TObject(lvKardex.Selected.Data).GetInterface(ICPKardexAction, aAction) then
      if aAction.getStatus = asActive then
        if SelectDate(aNewDate, aAction.getStopDateTime, 'Renew Action Thru') then
          MyCPKardex.RenewAction(aAction, MyUser, IncMinute(trunc(aNewDate), 1439));
  acRefresh.Execute;
end;

procedure TfraTaskView.acUnHoldExecute(Sender: TObject);
var
  i: Integer;
//  j: integer;
  aAction: ICPKardexAction;
//  aEvent: ICPKardexEvent;
  aUnHoldDate: TDateTime;
  aComment: string;
begin
  aUnHoldDate := Now;
  aComment := '';
  for i := 0 to lvKardex.Items.Count - 1 do
    if lvKardex.Items[i].Selected then
      if TObject(lvKardex.Items[i].Data).GetInterface(ICPKardexAction, aAction) then
        if aAction.getStatus = asHeld then
          if getKardexHoldDateAndComment(aAction, aUnHoldDate, aComment) then
            MyCPKardex.UnHoldAction(aAction, MyUser, aUnHoldDate, aComment);
  acRefresh.Execute;
end;

constructor TfraTaskView.Create(AOwner: TComponent);
var
  Item: TMenuItem;
begin
  inherited;
  FMenuItem := TMenuItem.Create(nil);
  FMenuItem.Caption := 'Kardex';

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acRefresh;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acNew;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acRenew;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acDiscontinue;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acHold;

  Item := TMenuItem.Create(FMenuItem);
  FMenuItem.Add(Item);
  Item.Action := acUnHold;
end;

destructor TfraTaskView.Destroy;
begin
  FreeAndNil(FMenuItem);
  inherited;
end;

procedure TfraTaskView.lvKardexSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);

  function AllAreSameStatus(aStatus: TCPKardexActionStatus): boolean;
  var
    i: integer;
    aAction: ICPKardexAction;
  begin
    Result := False;

    if lvKardex.SelCount = 0 then
      Exit;

    for i := 0 to lvKardex.Items.Count - 1 do
      if lvKardex.Items[i].Selected then
        if TObject(lvKardex.Items[i].Data).GetInterface(ICPKardexAction, aAction) then
          if aAction.getStatus <> aStatus then
            Exit;

    Result := True;
  end;

begin
  acRenew.Enabled := AllAreSameStatus(asActive);
  acDiscontinue.Enabled := AllAreSameStatus(asActive);
  acHold.Enabled := AllAreSameStatus(asActive);
  acUnHold.Enabled := AllAreSameStatus(asHeld);
end;

procedure TfraTaskView.ViewActivate;
begin
  Visible := True;
  ViewReload;
end;

procedure TfraTaskView.ViewDeactivate;
begin
  Visible := False;
end;

function TfraTaskView.ViewHelpContext: integer;
begin
  result := HelpContext;
end;

procedure TfraTaskView.ViewInitialize;
begin
  //
end;

procedure TfraTaskView.ViewReload;
var
  i: integer;
  aAction: ICPKardexAction;
begin
  MyCPKardex.Load(MyPatient, MyStartDateTime, MyStopDateTime);
  lvKardex.Items.BeginUpdate;
  try
    MyCPKardex.LoadActions(lvKardex);
    for i := 0 to lvKardex.Items.Count - 1 do
      with lvKardex.Items[i] do
        if TObject(Data).GetInterface(ICPKardexAction, aAction) then
          begin
            SubItems.Clear;
            Caption := aAction.getCareAction.getTerm;
            if aAction.getSchedule <> nil then
              SubItems.Add(aAction.getSchedule.getName)
            else
              SubItems.Add(CP_SCHEDULE_TYPE[aAction.getScheduleType]);
            SubItems.Add(CP_KARDEX_ACTION_STATUS[aAction.getStatus]);
            SubItems.Add(FormatDateTime('YYYY-MM-DD hhmm', aAction.getStartDateTime));
            SubItems.Add(FormatDateTime('YYYY-MM-DD hhmm', aAction.getStopDateTime));
            SubItems.Add(aAction.getSpecialInstructions);
          end;
  finally
    lvKardex.Items.EndUpdate;
  end;
end;

function TfraTaskView.ViewMenu: TMenuItem; stdcall;
begin
  Result := FMenuItem;
end;

end.

