unit uCP508;

interface

uses
  ActnList,
  Controls,
  ComCtrls,
  Menus,
  StdCtrls,
  SysUtils,
  VA508AccessibilityManager;

procedure Add508Component(aComponent: TWinControl; aLabel: TLabel); overload;
procedure Add508Component(aComponent: TWinControl; aText: string); overload;
procedure Remove508Component(aComponent: TWinControl);

function Build508ToolBarText(aToolBar: TToolBar): string;

function get508ComponentDisplay: string;

var
  MyVA508Mgr: TVA508AccessibilityManager;

implementation

uses Classes;

procedure Add508Component(aComponent: TWinControl; aLabel: TLabel); overload;
begin
  aComponent.TabStop := True;
  if Assigned(MyVA508Mgr) then
    with MyVA508Mgr.AccessData.FindItem(aComponent, True) do
      begin
        //UseDefault := False;
        AccessLabel := aLabel;
      end;
end;

procedure Add508Component(aComponent: TWinControl; aText: string); overload;
var
  aItem: TVA508AccessibilityItem;
begin
  aComponent.TabStop := True;
  if Assigned(MyVA508Mgr) then
    begin
      //MyVA508Mgr.AccessText[aComponent] := aText;
      (*   *)
      aItem := MyVA508Mgr.AccessData.FindItem(aComponent);
      if Assigned(aItem) then
        begin
          //aItem.UseDefault := False;
          aItem.AccessText := aText;
          //aItem.UseDefault := False;
        end;
      (*  *)
    end;
end;

procedure Remove508Component(aComponent: TWinControl);
var
  aItem: TVA508AccessibilityItem;
begin
  if Assigned(MyVA508Mgr) then
    begin
      aItem := MyVA508Mgr.AccessData.FindItem(aComponent, False);
      if Assigned(aItem) then
        MyVA508Mgr.AccessData.Delete(aItem.Index);
    end;
end;

function Build508ToolBarText(aToolBar: TToolBar): string;
var
  i: integer;
  aCount: integer;
begin
  Result := aToolBar.Caption + #13;
  aCount := 0;
  for i := 0 to aToolBar.ButtonCount - 1 do
    if aToolBar.Buttons[i].Visible then
      begin
        inc(aCount);
        if Assigned(aToolBar.Buttons[i].Action) then
          with TAction(aToolBar.Buttons[i].Action) do
            begin
              Result := Format('%s Button %d - %s', [Result, aCount, Caption]);
              if Hint <> '' then
                Result := Format('%s %s', [Result, Hint]);

              //if ShortCut <> nil then
              Result := Format('%s Shortcut Key is %s', [Result, ShortCutToText(ShortCut)]);
            end
        else
          begin
            Result := Format('%s Button %d - %s', [Result, aCount, aToolBar.Buttons[i].Caption]);
          end;
        Result := Result + #13;
      end;
end;

function get508ComponentDisplay: string;
var
  i: integer;
begin
  Result := Format('Component Count %d', [MyVA508Mgr.AccessData.Count]);
  for i := 0 to MyVA508Mgr.AccessData.Count - 1 do
    with MyVA508Mgr.AccessData[i] do
      begin
        if UseDefault then
          Result := Result + #13 + intToStr(i) + '  ' + Component.Name + ' /true/ ' + AccessProperty + ' / ' + AccessText
        else
          Result := Result + #13 + intToStr(i) + '  ' + Component.Name + ' /false/ ' + AccessProperty + ' / ' + AccessText;
      end;
end;

end.

